from typing import TYPE_CHECKING
import re
import time as ti

from .type import Update

if TYPE_CHECKING:
    from .type import Update

class Filter:
    def __call__(self, update: 'Update') -> bool:
        raise NotImplementedError

class text(Filter):
    """filter text message by text /  فیلتر کردن متن پیام بر اساس متنی"""
    def __init__(self, pattern: str):
        self.pattern = pattern

    def __call__(self, update: 'Update') -> bool:
        return update.text == self.pattern

class sender_id(Filter):
    """filter guid message by guid / فیلتر کردن شناسه گوید پیام"""
    def __init__(self, user_id: str):
        self.user_id = user_id

    def __call__(self, update: 'Update') -> bool:
        return update.message.get('sender_id') == self.user_id

class is_user(Filter):
    """filter type sender message by is PV(user) / فیلتر کردن تایپ ارسال کننده پیام با پیوی"""
    def __call__(self, update: 'Update') -> bool:
        return update.sender_type == "User"

class is_group(Filter):
    """filter type sender message by is group / فیلتر کردن تایپ ارسال کننده پیام با گروه"""
    def __call__(self, update: 'Update') -> bool:
        return update.sender_type == "Group"

class is_channel(Filter):
    """filter type sender message by is channel / فیلتر کردن تایپ ارسال کننده پیام با کانال"""
    def __call__(self, update: 'Update') -> bool:
        return update.sender_type == "Channel"

class is_file(Filter):
    """filter by file / فیلتر با فایل"""
    def __call__(self, update:'Update'):
        return True if update.file else False

class file_name(Filter):
    """filter by name file / فیلتر با اسم فایل"""
    def __init__(self,name_file: str):
        self.name_file = name_file
    def __call__(self, update:'Update'):
        return True if update.file_name==self.name_file else False

class size_file(Filter):
    """filter by name file / فیلتر با اسم فایل"""
    def __init__(self,size: int):
        self.size = size
    def __call__(self, update:'Update'):
        return True if update.size_file==self.size else False

class is_video(Filter):
    """filter by video / فیلتر با ویدیو"""
    def __call__(self, update:'Update'):
        return True if update.type_file=="video" else False

class is_image(Filter):
    """filter by image / فیلتر با عکس"""
    def __call__(self, update:'Update'):
        return True if update.type_file=="image" else False

class is_audio(Filter):
    """filter by audio / فیلتر با آودیو"""
    def __call__(self, update:'Update'):
        return True if update.type_file=="audio" else False

class is_voice(Filter):
    """filter by voice / فیلتر با ویس"""
    def __call__(self, update:'Update'):
        return True if update.type_file=="voice" else False

class is_document(Filter):
    """filter by document / فیلتر با داکیومنت"""
    def __call__(self, update:'Update'):
        return True if update.type_file=="document" else False

class is_web(Filter):
    """filter by web files / فیلتر با فایل های وب"""
    def __call__(self, update:'Update'):
        return True if update.type_file=="web" else False

class is_code(Filter):
    """filter by code files / فیلتر با فایل های کد"""
    def __call__(self, update:'Update'):
        return True if update.type_file=="code" else False

class is_archive(Filter):
    """filter by archive files / فیلتر با فایل های آرشیو"""
    def __call__(self, update:'Update'):
        return True if update.type_file=="archive" else False

class is_executable(Filter):
    """filter by executable files / فیلتر با فایل های نصبی"""
    def __call__(self, update:'Update'):
        return True if update.type_file=="executable" else False

class is_text(Filter):
    """filter by had text / فیلتر با داشتن متن"""
    def __call__(self, update:'Update'):
        return True if update.text!=None else False

class regex(Filter):
    """filter text message by regex pattern / فیلتر متن پیام با regex"""
    def __init__(self, pattern: str, flags=0):
        self.pattern = re.compile(pattern, flags)
    def __call__(self, update: 'Update') -> bool:
        if not hasattr(update, "text") or update.text is None:
            return False
        return bool(self.pattern.search(update.text))

class time(Filter):
    """filter by time / فیلتر با زمان"""
    def __init__(self,from_time:float=0,end_time=float("inf")):
        self.from_time = from_time
        self.end_time = end_time
    def __call__(self,update:'Update'):
        if ti.time()>self.from_time and ti.time()<self.end_time:
            return True
        return False



class commands(Filter):
    """filter text message by commands / فیلتر کردن متن پیام با دستورات"""
    def __init__(self, coms: list):
        self.coms = coms

    def __call__(self, update: 'Update') -> bool:
        for txt in self.coms:
            if (update.text!=None) and (update.text==txt or update.text.replace("/","")==txt):
                return True
        return False

class author_guids(Filter):
    """filter guid message by guids / فیلتر کردن گوید پیام با گوید ها"""
    def __init__(self, guids: list):
        self.guids = guids

    def __call__(self, update: 'Update') -> bool:
        for g in self.guids:
            if update.sender_id==g:
                return True
        return False

class chat_ids(Filter):
    """filter chat_id message by chat ids / فیلتر کردن چت آیدی پیام ارسال شده با چت آیدی ها"""
    def __init__(self, ids: list):
        self.ids = ids

    def __call__(self, update: 'Update') -> bool:
        for c in self.ids:
            if update.chat_id==c:
                return True
        return False

class has_metadata_type(Filter):
    def __init__(self,type) -> None:
        self.type = type
    def __call__(self, update: 'Update') -> bool:
        if update.meta_data_parts:
            for mata_data in update.meta_data_parts.data:
                if mata_data["type"].lower() == self.type.lower():
                    return True
        return False

class is_metadata_type(Filter):
    def __init__(self,type) -> None:
        self.type = type
    def __call__(self, update: 'Update') -> bool:
        if update.meta_data_parts and len(update.meta_data_parts.data) != 1:
            if update.meta_data_parts.data[0]["type"].lower() == self.type.lower():
                return True
        return False

class has_bold(Filter):
    """check for has bold text / چک وجود داشتن متن بولد"""
    def __call__(self, update: 'Update') -> bool:
        return has_metadata_type("bold")(update)

class has_italic(Filter):
    """check for has italic text / چک وجود داشتن متن ایتالیک"""
    def __call__(self, update: 'Update') -> bool:
        return has_metadata_type("italic")(update)

class has_underline(Filter):
    """check for has underline text / چک وجود داشتن متن آندرلایبن"""
    def __call__(self, update: 'Update') -> bool:
        return has_metadata_type("underline")(update)

class has_strike(Filter):
    """check for has strike text / چک وجود داشتن متن خط خورده"""
    def __call__(self, update: 'Update') -> bool:
        return has_metadata_type("strike")(update)

class has_mono(Filter):
    """check for has mono text / چک وجود داشتن متن کپی"""
    def __call__(self, update: 'Update') -> bool:
        return has_metadata_type("mono")(update)

class has_spoiler(Filter):
    """check for has spoiler text / چک وجود داشتن متن اسپویلر"""
    def __call__(self, update: 'Update') -> bool:
        return has_metadata_type("spoiler")(update)

class has_link(Filter):
    """check for has link text / چک وجود داشتن متن هایپر لینک"""
    def __call__(self, update: 'Update') -> bool:
        return has_metadata_type("link")(update)

class is_bold(Filter):
    """all text is bold / بولد بودن تمام متن"""
    def __call__(self, update: 'Update') -> bool:
        return is_metadata_type("bold")(update)

class is_italic(Filter):
    """all text is italic / ایتالیک بودن تمام متن"""
    def __call__(self, update: 'Update') -> bool:
        return is_metadata_type("italic")(update)

class is_underline(Filter):
    """all text is underline / آندرلاین بودن تمام متن"""
    def __call__(self, update: 'Update') -> bool:
        return is_metadata_type("underline")(update)

class is_strike(Filter):
    """all text is strike / خط خورده بودن تمام متن"""
    def __call__(self, update: 'Update') -> bool:
        return is_metadata_type("strike")(update)

class is_mono(Filter):
    """all text is mono / متن کپی بودن تمام متن"""
    def __call__(self, update: 'Update') -> bool:
        return is_metadata_type("mono")(update)

class is_spoiler(Filter):
    """all text is spoiler / اسپویلر بودن تمام متن"""
    def __call__(self, update: 'Update') -> bool:
        return is_metadata_type("spoiler")(update)

class is_link(Filter):
    """all text is link / هایپر لینک بودن تمام متن"""
    def __call__(self, update: 'Update') -> bool:
        return is_metadata_type("link")(update)


class in_text(Filter):
    """text in text message / وجود متن در متن آپدیت"""
    def __init__(self,text: str) -> None:
        self.text = text
    def __call__(self, update: 'Update') -> bool:
        if self.text in str(update.text):
            return True
        return False

class is_forward(Filter):
    """message is forward / پیام فوروارد شده"""
    def __call__(self, update: 'Update') -> bool:
        return update.is_fowrard

class is_reply(Filter):
    """message has reply / پیام دارای ریپلای"""
    def __call__(self, update: Update) -> bool:
        return update.reply_to_message_id != None

class text_length(Filter):
    """filter by text length / فیلتر بر اساس طول متن"""
    def __init__(self, min_len: int = 0, max_len: float = float('inf')):
        self.min_len = min_len
        self.max_len = max_len
    def __call__(self, update: 'Update') -> bool:
        if update.text:
            return self.min_len <= len(update.text) <= self.max_len
        return False

class starts_with(Filter):
    """filter text starting with / فیلتر متن هایی که با این شروع میشن"""
    def __init__(self, prefix: str):
        self.prefix = prefix
    def __call__(self, update: 'Update') -> bool:
        return update.text != None and update.text.startswith(self.prefix)

class ends_with(Filter):
    """filter text ending with / فیلتر متن هایی که با این پایان میابند"""
    def __init__(self, suffix: str):
        self.suffix = suffix
    def __call__(self, update: 'Update') -> bool:
        return update.text != None and update.text.endswith(self.suffix)

class is_sticker(Filter):
    """filter by sticker / فیلتر استیکر"""
    def __call__(self, update: 'Update') -> bool:
        return update.is_sticker

class is_contact(Filter):
    """filter by contact / فیلتر مخاطب"""
    def __call__(self, update: 'Update') -> bool:
        return update.is_contact



class and_filter(Filter):
    """filters {and} for if all filters is True : run code ... / فیلتر های ورودی {and} که اگر تمامی فیلتر های ورودی برابر True بود اجرا شود"""
    def __init__(self, *filters):
        self.filters = filters

    def __call__(self, update: 'Update') -> bool:
        return all(f(update) for f in self.filters)

class or_filter(Filter):
    """filters {or} for if one filter is True : run code ... / فیلتر های ورودی {and} که اگر یک فیلتر ورودی برابر True بود اجرا شود"""
    def __init__(self, *filters):
        self.filters = filters

    def __call__(self, update: 'Update') -> bool:
        return any(f(update) for f in self.filters)

class not_filter(Filter):
    """not True filter / درست نبودن فیلتر"""
    def __init__(self,filter):
        self.filter = filter
    def __call__(self, update: Update) -> bool:
        return not self.filter(update)


from .. import *
from ..core.async_sync import *
from .get_type import *
from .props import *
from .metadata import metadata as metadata_prop
from .file import file as FILE
import json
from pathlib import Path
from typing import TYPE_CHECKING, Optional, Union
if TYPE_CHECKING:
    from .prop_update import msg_update

class Update:
    def __init__(self, update_data: dict, client: "Client"):
        try:
            self._data = update_data["new_message"]
        except:
            self._data = update_data["updated_message"]
        self._client = client
        self.raw_data_ = update_data
        self.message = update_data.get("new_message", {})
    @property
    def text(self) -> Optional[str]:
        """text message / متن پیام"""
        return self._data['text'] if "text" in self._data else None
    @property
    def message_id(self) -> str:
        """message id / آیدی پیام"""
        return self._data['message_id']
    @property
    def chat_id(self) -> str:
        """chat id message / چت آیدی پیام"""
        return self.raw_data_['chat_id']
    @property
    def time(self) -> int:
        """time sended message / زمان ارسال شده پیام"""
        return int(self._data['time'])
    @property
    def sender_type(self) -> Literal["User","Group","Channel"]:
        """sender type / نوع ارسال کننده"""
        if self.chat_id.startswith("b"):
            return "User"
        elif self.chat_id.startswith("g"):
            return "Group"
        elif self.chat_id.startswith("c"):
            return "Channel"
        else:
            raise ValueError("chat id is not found")
    @property
    def sender_id(self) -> str:
        """sender id message / شناسه گوید کاربر ارسال کننده"""
        return self._data['sender_id']
    @property
    def is_edited(self):
        return self._data['is_edited']
    @property
    def file(self) -> Optional[dict]:
        """file / فایل"""
        return self._data['file'] if "file" in self._data else None
    @property
    def file_id(self) -> Optional[str]:
        """file id / آیدی فایل"""
        return self._data['file']['file_id'] if "file" in self._data else None
    @property
    def file_name(self) -> Optional[str]:
        """file name / اسم فایل"""
        return self._data['file']['file_name'] if "file" in self._data else None
    @property
    def size_file(self) -> Optional[int]:
        """size file / سایز فایل"""
        return self._data['file']['size'] if "file" in self._data else None
    @property
    def type_file(self) -> str:
        """get type file / گرفتن نوع فایل"""
        if self.file_name:
            return get_file_category(self.file_name)
        else:
            return "text"
    @property
    def button(self) -> Optional[dict]:
        """data button clicked / اطلاعات دکمه کلیک شده"""
        return self._data['aux_data'] if "aux_data" in self._data else None
    @property
    def button_id(self) -> Optional[str]:
        """button id clicked button / آیدی دکمه کلیک شده"""
        return self.button['button_id'] if self.button else None
    @property
    def reply_to_message_id(self) -> Optional[str]:
        """message id replyed / آیدی پیام ریپلای شده"""
        return self._data["reply_to_message_id"] if "reply_to_message_id" in self._data else None
    @property
    def metadata(self) -> Optional[dict]:
        """meta data / متا دیتا"""
        return self._data["metadata"] if "metadata" in self._data else None
    @property
    def meta_data_parts(self) -> Optional[metadata_prop]:
        """meta data parts list / لیست قسمت های متا دیتا"""
        return metadata_prop(self.metadata["meta_data_parts"]) if self.metadata else None
    @property
    def is_fowrard(self) -> bool:
        """forwarded / فوروارد شده"""
        return "forwarded_from" in self._data
    @property
    def forward_from(self) -> Optional[str]:
        """forward from / فوروارد از"""
        if self.is_fowrard:
            return self._data["forwarded_from"]["type_from"]
        return None
    @property
    def forward_message_id(self) -> Optional[str]:
        """message id forward / آیدی پیام فوروارد شده"""
        if self.is_fowrard:
            return self._data["forwarded_from"]["message_id"]
        return None
    @property
    def forward_from_sender_id(self) -> Optional[str]:
        """sender id forwarded / شناسه گوید فوروارد کننده"""
        if self.is_fowrard:
            return self._data["forwarded_from"]["from_sender_id"]
        return None
    @property
    def is_contact(self) -> bool:
        """contect / مخاطب"""
        return "contact_message" in self._data
    @property
    def contact_phone_number(self) -> Optional[str]:
        """phone number contact / شماره همراه مخاطب"""
        if self.is_contact:
            return self._data["contact_message"]["phone_number"]
        return None
    @property
    def contact_first_name(self) -> Optional[str]:
        """first name contact / اسم مخاطب"""
        if self.is_contact:
            return self._data["contact_message"]["first_name"]
        return None
    @property
    def contact_last_name(self) -> Optional[str]:
        """last name contact / نام خانوادگی مخاطب"""
        if self.is_contact:
            return self._data["contact_message"]["last_name"]
        return None
    @property
    def is_sticker(self) -> bool:
        """sticker / استیکر"""
        return "sticker" in self._data
    @property
    def sticker_emoji_character(self) -> Optional[str]:
        """imoji sticker character / کاراکتر ایموجی استیکر"""
        if self.is_sticker:
            return self._data["sticker"]["emoji_character"]
        return None
    @property
    def sticker_sticker_id(self) -> Optional[str]:
        """sticker id / آیدی استیکر"""
        if self.is_sticker:
            return self._data["sticker"]["sticker_id"]
        return None
    @property
    def sticker_file(self) -> Union[FILE,dict]:
        """file sticker / فایل استیکر"""
        if self.is_sticker:
            return FILE(self._data["sticker"]["file"])
        return {}


    @auto_async
    async def get_chat_id_info(self) -> props:
        """get info the chat id / گرفتن درباره چت آیدی"""
        return await self._client.get_chat(self.chat_id)

    @auto_async
    async def reply(
        self,
        text: str,
        keypad_inline: Optional[dict] = None,
        keypad: Optional[dict] = None,
        resize_keyboard: bool | None = True,
        on_time_keyboard: bool | None = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal['Markdown', 'HTML', None] = "Markdown"
    ) -> 'msg_update':
        """reply text / ریپلای متن"""
        return await self._client.send_text(
            text=text, chat_id=self.chat_id, reply_to_message_id=self.message_id, inline_keypad=keypad_inline, auto_delete=auto_delete, parse_mode=parse_mode,keypad=keypad,on_time_keyboard=on_time_keyboard,resize_keyboard=resize_keyboard
        )

    @auto_async
    async def reply_poll(
        self,
        question: str,
        options: list,
        type_poll: Literal["Regular", "Quiz"] = "Regular",
        is_anonymous: bool = True,
        correct_option_index: Optional[int] = None,
        allows_multiple_answers: bool = False,
        hint: Optional[str] = None,
        auto_delete: Optional[int] = None
    ) -> props:
        """reply poll / ریپلای نظرسنجی"""
        return await self._client.send_poll(self.chat_id, question, options,type_poll=type_poll,is_anonymous=is_anonymous,correct_option_index=correct_option_index,allows_multiple_answers=allows_multiple_answers,hint=hint,auto_delete=auto_delete,reply_to_message_id=self.message_id)

    @auto_async
    async def reply_contact(
        self, first_name: str, phone_number: str, last_name: Union[str,str] = "",auto_delete: Optional[int] = None
    ) -> props:
        """reply contact / ریپلای مخاطب"""
        return await self._client.send_contact(
            self.chat_id,
            first_name,
            last_name,
            phone_number,
            reply_to_message_id=self.message_id,
            auto_delete=auto_delete
        )

    @auto_async
    async def reply_location(self, latitude: str, longitude: str,auto_delete: Optional[int] = None) -> props:
        """reply location / ریپلای موقعیت مکانی"""
        return await self._client.send_location(
            self.chat_id, latitude, longitude, reply_to_message_id=self.message_id,auto_delete=auto_delete
        )

    @auto_async
    async def reply_file(
        self,
        file: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text: Optional[str] = None,
        type_file: Literal["File", "Image", "Voice", "Music", "Gif","Video"] = "File",
        disable_notification: Optional[bool] = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal['Markdown', 'HTML', None] = "Markdown"
    ) -> props:
        """reply file / ریپلای فایل"""
        return await self._client.base_send_file(
            self.chat_id,
            file,
            name_file,
            text,
            self.message_id,
            type_file,
            disable_notification,
            auto_delete,
            parse_mode
        )

    @auto_async
    async def reply_image(
        self,
        image: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text: Optional[str] = None,
        disable_notification: Optional[bool] = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal['Markdown', 'HTML', None] = "Markdown"
    ) -> props:
        """reply image / رپیلای تصویر"""
        return await self._client.send_image(
            self.chat_id, image, name_file, text, self.message_id, disable_notification,auto_delete,parse_mode
        )

    @auto_async
    async def reply_voice(
        self,
        voice: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text: Optional[str] = None,
        disable_notification: Optional[bool] = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal['Markdown', 'HTML', None] = "Markdown"
    ) -> props:
        """reply voice / رپیلای ویس"""
        return await self._client.send_voice(
            self.chat_id, voice, name_file, text, self.message_id, disable_notification,auto_delete,parse_mode
        )

    @auto_async
    async def reply_music(
        self,
        music: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text: Optional[str] = None,
        disable_notification: Optional[bool] = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal['Markdown', 'HTML', None] = "Markdown"
    ) -> props:
        """reply voice / رپیلای موزیک"""
        return await self._client.send_music(
            chat_id=self.chat_id, music=music, name_file=name_file, text=text, reply_to_message_id=self.message_id, disable_notification=disable_notification,auto_delete=auto_delete,parse_mode=parse_mode
        )

    @auto_async
    async def reply_gif(
        self,
        gif: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text: Optional[str] = None,
        disable_notification: Optional[bool] = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal['Markdown', 'HTML', None] = "Markdown"
    ) -> props:
        """reply voice / رپیلای گیف"""
        return await self._client.send_gif(
            self.chat_id, gif, name_file, text, self.message_id, disable_notification,auto_delete,parse_mode
        )

    @auto_async
    async def reply_video(
        self,
        video: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text: Optional[str] = None,
        disable_notification: Optional[bool] = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal['Markdown', 'HTML', None] = "Markdown"
    ) -> props:
        """reply voice / رپیلای ویدیو"""
        return await self._client.send_video(
            self.chat_id, video, name_file, text, self.message_id, disable_notification,auto_delete,parse_mode
        )

    @auto_async
    async def forward(
            self,
            to_chat_id:str,
            auto_delete: Optional[int] = None
    ) -> props:
        """forward / فوروارد"""
        return await self._client.forward_message(self.chat_id,self.message_id,to_chat_id,auto_delete=auto_delete)

    @auto_async
    async def download(
            self,
            path : Union[str,str] = "file"
    ) -> Optional[dict]:
        """download / دانلود"""
        if self.file_id:
            return await self._client.download_file(self.file_id,path)
        return None

    @auto_async
    async def delete(
            self
    ) -> props:
        """delete / حذف"""
        return await self._client.delete_message(self.chat_id,self.message_id)

    def __str__(self) -> str:
        if self.file_name:
            self._data['file']['type']=self.type_file
        self._data["sender_type"]=self.sender_type
        return json.dumps(self._data,indent=4,ensure_ascii=False)

    def __repr__(self) -> str:
        return self.__str__()


class UpdateButton:
    def __init__(self, data: dict,client: "Client"):
        self._data = data
        self._client = client

    @property
    def raw_data(self) -> dict:
        return self._data

    @property
    def button_id(self) -> str:
        """button id clicked / آیدی دکمه کلیک شده"""
        return self._data["inline_message"]["aux_data"]["button_id"]

    @property
    def chat_id(self) -> str:
        """chat id clicked / چت آیدی کلیک شده"""
        return self._data["inline_message"]["chat_id"]

    @property
    def message_id(self) -> str:
        """message id for message clicked glass button / آیدی پیام کلیک شده روی دکمه شیشه ای"""
        return self._data["inline_message"]["message_id"]

    @property
    def sender_id(self) -> str:
        """guid for clicked button glass / شناسه گوید کاربر کلیک کرده روی دکمه شیشه ای"""
        return self._data["inline_message"]["sender_id"]

    @property
    def text(self) -> str:
        """text for button clicked / متن دکمه شیشه ای که روی آن کلیک شده"""
        return self._data["inline_message"]["text"]

    @auto_async
    async def send_text(self,text:str,inline_keypad: Optional[dict] = None,keypad: Optional[dict] = None,resize_keyboard: Optional[bool] = True,on_time_keyboard: Optional[bool] = False,auto_delete: Optional[int] = None,reply_to_message_id: Optional[str] = None,parse_mode: Literal['Markdown', 'HTML'] = "Markdown"):
        return await self._client.send_text(
            text, self.chat_id,inline_keypad=inline_keypad,auto_delete=auto_delete,reply_to_message_id=reply_to_message_id,parse_mode=parse_mode,keypad=keypad,resize_keyboard=resize_keyboard,on_time_keyboard=on_time_keyboard
        )

    @auto_async
    async def send_pool(
        self,
        question: str,
        options : list,
        type_poll: Literal['Regular', 'Quiz'] = "Regular",
        is_anonymous: bool = True,
        correct_option_index: int | None = None,
        allows_multiple_answers: bool = False,
        hint: str | None = None,
        auto_delete: Optional[int] = None
    ):
        return await self._client.send_poll(
            chat_id=self.chat_id,question=question,options=options,type_poll=type_poll,is_anonymous=is_anonymous,correct_option_index=correct_option_index,allows_multiple_answers=allows_multiple_answers,hint=hint,disable_notification=False,auto_delete=auto_delete
        )

    @auto_async
    async def send_contact(
        self,
        first_name: str,
        last_name: str,
        phone_number: str,
        auto_delete: Optional[int] = None,
        reply_to_message_id: Optional[str] = None
    ):
        return await self._client.send_contact(
            self.chat_id,first_name,last_name,phone_number,auto_delete=auto_delete,reply_to_message_id=reply_to_message_id
        )

    @auto_async
    async def send_location(
        self,
        latitude: str,
        longitude: str,
        auto_delete: Optional[int] = None,
        reply_to_message_id: Optional[str] = None
    ):
        return await self._client.send_location(
            self.chat_id,latitude,longitude,auto_delete=auto_delete,reply_to_message_id=reply_to_message_id
        )

    @auto_async
    async def send_file(
        self,
        file: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text: Optional[str] = None,
        type_file: Literal['File', 'Image', 'Voice', 'Music', 'Gif', 'Video'] = "File",
        auto_delete: Optional[int] = None,
        reply_to_message_id: Optional[str] = None
    ):
        return await self._client.base_send_file(
            self.chat_id,file,name_file,text,type_file=type_file,auto_delete=auto_delete,reply_to_message_id=reply_to_message_id
        )

    @auto_async
    async def send_image(
        self,
        image: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text: Optional[str] = None,
        auto_delete: Optional[int] = None,
        reply_to_message_id: Optional[str] = None
    ):
        return await self._client.send_image(
            self.chat_id,
            image,
            name_file,
            text,
            auto_delete=auto_delete,
            reply_to_message_id=reply_to_message_id
        )

    @auto_async
    async def send_video(
        self,
        video: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text: Optional[str] = None,
        auto_delete: Optional[int] = None,
        reply_to_message_id: Optional[str] = None
    ):
        return await self._client.send_video(
            self.chat_id,
            video,
            name_file,
            text,
            auto_delete=auto_delete,
            reply_to_message_id=reply_to_message_id
        )

    @auto_async
    async def send_voice(
        self,
        voice: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text: Optional[str] = None,
        auto_delete: Optional[int] = None,
        reply_to_message_id: Optional[str] = None
    ):
        return await self._client.send_voice(
            self.chat_id,
            voice,
            name_file,
            text,
            auto_delete=auto_delete,
            reply_to_message_id=reply_to_message_id
        )

    @auto_async
    async def send_music(
        self,
        music: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text: Optional[str] = None,
        auto_delete: Optional[int] = None,
        reply_to_message_id: Optional[str] = None
    ):
        return await self._client.send_music(
            self.chat_id,
            music,
            name_file,
            text,
            auto_delete=auto_delete,
            reply_to_message_id=reply_to_message_id
        )

    @auto_async
    async def send_gif(
        self,
        gif: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text: Optional[str] = None,
        auto_delete: Optional[int] = None,
        reply_to_message_id: Optional[str] = None
    ):
        return await self._client.send_gif(
            self.chat_id,
            gif,
            name_file,
            text,
            auto_delete=auto_delete,
            reply_to_message_id=reply_to_message_id
        )

    def __str__(self) -> str:
        return json.dumps(self._data)


from fast_rub import Client
import asyncio

bot = Client("test")

chat_id = "b..."

async def test():
    sending = await bot.send_location(chat_id, "35.6892", "51.3890")
    print(sending)

asyncio.run(test())


from fast_rub import Client , filters
from fast_rub.type import Update

bot = Client("test")

@bot.on_message(filters.has_bold())
async def test1(msg:Update):
    await msg.reply("متن شما دارای بولد میباشد")

@bot.on_message(filters.regex("(hi | hello | سلام | درود)"))
async def test2(msg:Update):
    await msg.reply("درود")

@bot.on_message(filters.starts_with("+"))
async def test(msg:Update):
    # کد های برای مثال ارسال پیام برای هوش مصنوعی ... 
    text_gpt = "سلام خوبی" # برای مثال
    await msg.reply(text_gpt)

bot.run()


from fast_rub import Client
import asyncio

bot = Client("test")

from_chat_id = "b..."
message_id = "1234567890"
to_chat_id = "b..."

async def test():
    sending = await bot.forward_message(from_chat_id, message_id, to_chat_id)
    print(sending)

asyncio.run(test())


from random import randint
from ..network import (
    Network,
    Socket
)
from ..crypto import Cryption
from ..utils import Utils
from random import choice
from time import sleep
from ..exceptions import (
    InvalidAuth,
    InvalidInput
)
import asyncio
from typing import (
    Optional,
    Union,
    Literal
)
from ...core.async_sync import *
from typing import Optional, Union, List
from ..filters import Filter

class Methods:
    def __init__(self, sessionData:dict, platform:str, apiVersion:int, proxy:Optional[str], timeOut:int, showProgressBar:bool) -> None:
        self.platform = platform.lower()
        if not self.platform in ["android", "web", "rubx", "rubikax", "rubino"]:
            print("The \"{}\" is not a valid platform. Choose these one -> (web, android, rubx)".format(platform))
            exit()
        self.apiVersion = apiVersion
        self.proxy = proxy
        self.timeOut = timeOut
        self.showProgressBar = showProgressBar
        self.sessionData = sessionData
        self.crypto = Cryption(
            auth=sessionData["auth"],
            private_key=sessionData["private_key"]
        ) if sessionData else Cryption(auth=Utils.randomTmpSession())
        self.network = Network(methods=self)
        self.socket = Socket(methods=self)

    # Authentication methods

    @async_to_sync
    async def sendCode(self, phoneNumber:str, passKey:Optional[str] = None, sendInternal:bool=False) -> dict:
        input:dict = {
            "phone_number": f"{phoneNumber}",
            "send_type": "Internal" if sendInternal else "SMS",
        }

        if passKey:
            input["pass_key"] = passKey

        return await self.network.request(
            method="sendCode",
            input=input,
            tmpSession=True
        )
    
    @async_to_sync
    async def signIn(self, phoneNumber, phoneCodeHash, phoneCode) -> dict:
        publicKey, privateKey = self.crypto.rsaKeyGenrate()

        data = await self.network.request(
            method="signIn",
            input={
                "phone_number": f"98{Utils.phoneNumberParse(phoneNumber)}",
                "phone_code_hash": phoneCodeHash,
                "phone_code": phoneCode,
			    "public_key": publicKey
            },
            tmpSession=True
        )
        
        data["private_key"] = privateKey

        return data

    @async_to_sync
    async def registerDevice(self, deviceModel) -> dict:
        return await self.network.request(
            method="registerDevice",
            input={
                "app_version": "WB_4.3.3" if self.platform == "web" else "MA_3.4.3",
                "device_hash": Utils.randomDeviceHash(),
                "device_model": deviceModel,
                "is_multi_account": False,
                "lang_code": "fa",
                "system_version": "Windows 11" if self.platform == "web" else "SDK 28",
                "token": "",
                "token_type": "Web" if self.platform == "web" else "Firebase"
            }
        )

    @async_to_sync
    async def logout(self) -> dict:
        return await self.network.request(method="logout")
    
    # Chats methods
    
    @async_to_sync
    async def getChats(self, startId:Optional[str]) -> dict:
        return await self.network.request(method="getChats", input={"start_id": startId})
    
    @async_to_sync
    async def getObjectByUsername(self, username: str) -> dict:
        return await self.network.request(method="getObjectByUsername",input={'username': username.replace("@","")})
    
    @async_to_sync
    async def getTopChatUsers(self) -> dict:
        return await self.network.request(method="getTopChatUsers")
    
    @async_to_sync
    async def removeFromTopChatUsers(self, objectGuid:str) -> dict:
        return await self.network.request(method="removeFromTopChatUsers", input={"user_guid": objectGuid})
    
    @async_to_sync
    async def getChatAds(self) -> dict:
        return await self.network.request(method="getChatAds", input={"state": Utils.getState()})

    @async_to_sync
    async def getChatsUpdates(self) -> dict:
        return await self.network.request(method="getChatsUpdates", input={"state": Utils.getState()})
    
    @async_to_sync
    async def joinChat(self, guidOrLink:str) -> dict:
        if Utils.checkLink(guidOrLink):
            method:str = "joinGroup" if Utils.getChatTypeByLink(link=guidOrLink) == "Group" else "joinChannelByLink"
        else:
            method:str = "joinChannelAction"

        return await self.network.request(
            method=method,
            input={"hash_link": guidOrLink.split("/")[-1]} if Utils.checkLink(guidOrLink) else {
                "channel_guid": guidOrLink,
                "action": "Join"
            }
        )

    @async_to_sync
    async def actionOnJoinRequest(self,objectGuid: str, userGuid: str, action: Literal["Accept", "Reject"]):
        object_type = Utils.getChatTypeByGuid(objectGuid)
        return await self.network.request(
            method="actionOnJoinRequest",
            input={
                "object_guid": objectGuid,
                "object_type": object_type,
                "user_guid": userGuid,
                "action": action
            }
        )

    @async_to_sync
    async def getJoinRequests(self, objectGuid: str):
        return await self.network.request(
            method="getJoinRequests",
            input={
                "object_guid": objectGuid
            }
        )
    
    @async_to_sync
    async def leaveChat(self, objectGuid:str) -> dict:
        input:dict = {f"{Utils.getChatTypeByGuid(objectGuid=objectGuid).lower()}_guid": objectGuid}

        if Utils.getChatTypeByGuid(objectGuid=objectGuid) == "Group": method:str = "leaveGroup"
        else:
            method:str = "joinChannelAction"
            input["action"] = "Leave"

        return await self.network.request(
            method=method,
            input=input
        )
    
    @async_to_sync
    async def removeChat(self, objectGuid:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"remove{chatType}",
            input={f"{chatType.lower()}_guid": objectGuid}
        )
    
    @async_to_sync
    async def getChatInfo(self, objectGuid:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"get{chatType}Info",
            input={f"{chatType.lower()}_guid": objectGuid}
        )
    
    @async_to_sync
    async def getChatInfoByUsername(self, username:str) -> dict:
        return await self.network.request(method="getObjectInfoByUsername", input={"username": username.replace("@", "")})
    
    @async_to_sync
    async def getChatLink(self, objectGuid:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"get{chatType}Link",
            input={f"{chatType.lower()}_guid": objectGuid}
        )
    
    @async_to_sync
    async def setChatLink(self, objectGuid:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"set{chatType}Link",
            input={f"{chatType.lower()}_guid": objectGuid}
        )
    
    @async_to_sync
    async def setChatAdmin(self, objectGuid:str, memberGuid:str, accessList:Optional[list], customTitle:Optional[str], action:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        input:dict = {
            f"{chatType.lower()}_guid": objectGuid,
            "member_guid": memberGuid,
            "action": action,
            "access_list": accessList or []
        }

        if customTitle: input["custom_title"] = customTitle

        return await self.network.request(
            method=f"set{chatType}Admin",
            input=input
        )
    
    @async_to_sync
    async def addChatMember(self, objectGuid:str, memberGuids:list) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"add{chatType}Members",
            input={
                f"{chatType.lower()}_guid": objectGuid,
                "member_guids": memberGuids
            }
        )
    
    @async_to_sync
    async def banChatMember(self, objectGuid:str, memberGuid:str, action:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"ban{chatType}Member",
            input={
                f"{chatType.lower()}_guid": objectGuid,
                "member_guid": memberGuid,
                "action": action
            }
        )
    
    @async_to_sync
    async def getBannedChatMembers(self, objectGuid:str, startId:Optional[str]) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"getBanned{chatType}Members",
            input={
                f"{chatType.lower()}_guid": objectGuid,
                "start_id": startId
            }
        )
    
    @async_to_sync
    async def getChatAllMembers(self, objectGuid:str, searchText:Optional[str], startId:Optional[str], justGetGuids:bool=False) -> Union[dict,list]:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        data = await self.network.request(
            method=f"get{chatType}AllMembers",
            input={
                f"{chatType.lower()}_guid": objectGuid,
                "search_text": searchText.replace("@", "") if searchText else searchText,
                "start_id": startId
            }
        )

        if justGetGuids: return [i["member_guid"] for i in data["in_chat_members"]]

        return data
    
    @async_to_sync
    async def getChatAdminMembers(self, objectGuid:str, startId:Optional[str], justGetGuids: bool = False) -> Union[dict,list]:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        data = await self.network.request(
            method=f"get{chatType}AdminMembers",
            input={
                f"{chatType.lower()}_guid": objectGuid,
                "start_id": startId
            }
        )
    
        if justGetGuids: return [i["member_guid"] for i in data["in_chat_members"]]

        return data

    @async_to_sync
    async def userIsAdmin(self, objectGuid: str, userGuid: str):
        nextStartId = None
        hasContinue = True
        while hasContinue:
            result = await self.getChatAdminMembers(objectGuid, nextStartId)
            if type(result) is dict: # for typing error
                hasContinue = result["has_continue"]
                nextStartId = result["next_start_id"]
                for user in result["in_chat_members"]:
                    if userGuid == user.member_guid:
                        return True
        return False
    
    @async_to_sync
    async def getChatAdminAccessList(self, objectGuid:str, memberGuid:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"get{chatType}AdminAccessList",
            input={
                f"{chatType.lower()}_guid": objectGuid,
                "member_guid": memberGuid
            }
        )
    
    @async_to_sync
    async def chatPreviewByJoinLink(self, link:str) -> dict:
        return await self.network.request(
            method="groupPreviewByJoinLink" if "joing" in link else "channelPreviewByJoinLink",
            input={"hash_link": link.split("/")[-1]}
        )
    
    @async_to_sync
    async def createChatVoiceChat(self, objectGuid:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"create{chatType}VoiceChat",
            input={f"{chatType.lower()}_guid": objectGuid}
        )
    
    @async_to_sync
    async def joinVoiceChat(self, objectGuid:str, myGuid:str, voiceChatId:str, sdp_offer_data:str) -> dict:
        return await self.network.request(
            method=f"join{Utils.getChatTypeByGuid(objectGuid=objectGuid)}VoiceChat",
            input={
                "chat_guid": objectGuid,
                "voice_chat_id": voiceChatId,
                "sdp_offer_data": sdp_offer_data,
                "self_object_guid": myGuid
            }
        )

    @async_to_sync
    async def setChatVoiceChatSetting(self, objectGuid:str, voideChatId:str, title:Optional[str], joinMuted:Optional[bool]) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        input:dict = {
            f"{chatType.lower()}_guid": objectGuid,
            "voice_chat_id": voideChatId,
            "updated_parameters": []
        }

        if title:
            input["title"] = title
            input["updated_parameters"].append("title")
        
        if joinMuted:
            input["join_muted"] = joinMuted
            input["updated_parameters"].append("join_muted")

        return await self.network.request(
            method=f"set{chatType}VoiceChatSetting",
            input=input
        )
    
    @async_to_sync
    async def getChatVoiceChatUpdates(self, objectGuid:str, voideChatId:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"get{chatType}VoiceChatUpdates",
            input={
                f"{chatType.lower()}_guid": objectGuid,
                "voice_chat_id": voideChatId,
                "state": Utils.getState()
            }
        )
    
    @async_to_sync
    async def getChatVoiceChatParticipants(self, objectGuid:str, voideChatId:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"get{chatType}VoiceChatParticipants",
            input={
                f"{chatType.lower()}_guid": objectGuid,
                "voice_chat_id": voideChatId,
            }
        )
    
    @async_to_sync
    async def setChatVoiceChatState(self, objectGuid:str, voideChatId:str, activity:str, participantObjectGuid:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"set{chatType}VoiceChatState",
            input={
                "chat_guid": objectGuid,
                "voice_chat_id": voideChatId,
                "action": activity,
                "participant_object_guid": participantObjectGuid
            }
        )
    
    @async_to_sync
    async def sendChatVoiceChatActivity(self, objectGuid:str, voideChatId:str, activity:str, participantObjectGuid:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"send{chatType}VoiceChatActivity",
            input={
                "chat_guid": objectGuid,
                "voice_chat_id": voideChatId,
                "activity": activity,
                "participant_object_guid": participantObjectGuid
            }
        )
    
    @async_to_sync
    async def leaveChatVoiceChat(self, objectGuid:str, voideChatId:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"leave{chatType}VoiceChat",
            input={
                f"{chatType.lower()}_guid": objectGuid,
                "voice_chat_id": voideChatId
            }
        )
    
    @async_to_sync
    async def discardChatVoiceChat(self, objectGuid:str, voideChatId:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"discard{chatType}VoiceChat",
            input={
                f"{chatType.lower()}_guid": objectGuid,
                "voice_chat_id": voideChatId
            }
        )
    
    @async_to_sync
    async def setActionChat(self, objectGuid:str, action:str) -> dict:
        return await self.network.request(
            method="setActionChat",
            input={
                "object_guid": objectGuid,
                "action": action
            }
        )
    
    @async_to_sync
    async def seenChats(self, seenList:dict) -> dict:
        return await self.network.request(method="seenChats", input={"seen_list": seenList})
    
    @async_to_sync
    async def seenChatMessages(self, objectGuid:str, minId:str, maxId:str) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"seen{chatType}Messages",
            input={
                f"{chatType.lower()}_guid": objectGuid,
                "min_id": minId,
                "max_id": maxId
            }
        )
    
    @async_to_sync
    async def sendChatActivity(self, objectGuid:str, activity:str) -> dict:
        return await self.network.request(
            method="sendChatActivity",
            input={
                "object_guid": objectGuid,
                "activity": activity
            }
        )
    
    @async_to_sync
    async def searchChatMessages(self, objectGuid:str, searchText:str) -> dict:
        return await self.network.request(
            method="searchChatMessages",
            input={
                "object_guid": objectGuid,
                "search_text": searchText,
                "type": "Hashtag" if searchText.startswith("#") else "Text"
            }
        )
    
    @async_to_sync
    async def uploadAvatar(self, objectGuid:str, mainFile:str, thumbnailFile:Optional[str]) -> Optional[dict]:
        uploadMainFileData = self.network.upload(file=mainFile)

        if uploadMainFileData is not None and thumbnailFile is not None:
            uplo_ = self.network.upload(file=thumbnailFile)
            if isinstance(uplo_,dict):
                return await self.network.request(
                    method="uploadAvatar",
                    input={
                        "object_guid": objectGuid,
                        "thumbnail_file_id": uplo_["id"] if thumbnailFile else uploadMainFileData["id"],
                        "main_file_id": uploadMainFileData["id"]
                    }
                )
        raise ValueError("Error ! Value invalid")
    
    @async_to_sync
    async def getAvatars(self, objectGuid:str) -> dict:
        return await self.network.request(method="getAvatars", input={"object_guid": objectGuid})
    
    @async_to_sync
    async def deleteAvatar(self, objectGuid:str, avatarId:str) -> dict:
        return await self.network.request(
            method="deleteAvatar",
            input={
                "object_guid": objectGuid,
                "avatar_id": avatarId
            }
        )
    
    @async_to_sync
    async def deleteChatHistory(self, objectGuid:str, lastMessageId:str) -> dict:
        return await self.network.request(
            method="deleteChatHistory",
            input={
                "object_guid": objectGuid,
                "last_message_id": lastMessageId
            }
        )
    
    @async_to_sync
    async def deleteUserChat(self, userGuid:str, lastDeletedMessageId) -> dict:
        return await self.network.request(
            method="deleteUserChat",
            input={
                "user_guid": userGuid,
                "last_deleted_message_id": lastDeletedMessageId
            }
        )
    
    @async_to_sync
    async def getPendingObjectOwner(self, objectGuid:str) -> dict:
        return await self.network.request(method="getPendingObjectOwner", input={"object_guid": objectGuid})
    
    @async_to_sync
    async def requestChangeObjectOwner(self, objectGuid:str, memberGuid:str) -> dict:
        return await self.network.request(
            method="requestChangeObjectOwner",
            input={
                "new_owner_user_guid": memberGuid,
                "object_guid": objectGuid
            }
        )
    
    @async_to_sync
    async def replyRequestObjectOwner(self, objectGuid:str, action:str) -> dict:
        return await self.network.request(
            method="replyRequestObjectOwner",
            input={
                "action": action, #Accept Reject
                "object_guid": objectGuid
            }
        )
    
    @async_to_sync
    async def cancelChangeObjectOwner(self, objectGuid:str) -> dict:
        return await self.network.request(method="cancelChangeObjectOwner", input={"object_guid": objectGuid})
    
    @async_to_sync
    async def getChatReaction(self, objectGuid:str, minId:str, maxId:str) -> dict:
        return await self.network.request(
            method="getChatReaction",
            input={
                f"object_guid": objectGuid,
                "min_id": minId,
                "max_id": maxId
            }
        )
    
    @async_to_sync
    async def reportObject(self, objectGuid:str, description:str) -> dict:
        return await self.network.request(
            method="reportObject",
            input={
                "object_guid": objectGuid,
                "report_description": description,
                "report_type": 100,
                "report_type_object": "Object"
            }
        )
    
    @async_to_sync
    async def setChatUseTime(self, objectGuid:str, time:int) -> dict:
        return await self.network.request(
            method="setChatUseTime",
            input={
                "object_guid": objectGuid,
                "time": time
            }
        )
    
    # User methods

    @async_to_sync
    async def setBlockUser(self, objectGuid:str, action:str) -> dict:
        return await self.network.request(
            method="setBlockUser",
            input={
                "user_guid": objectGuid,
                "action": action
            }
        )
    
    @async_to_sync
    async def checkUserUsername(self, username:str) -> dict:
        return await self.network.request(
            method="checkUserUsername",
            input={
                "username": username,
            }
        )
    
    # Group methods

    @async_to_sync
    async def addGroup(self, title:str, memberGuids:list) -> dict:
        return await self.network.request(
            method="addGroup",
            input={
                "title": title,
                "member_guids": memberGuids
            }
        )
    
    @async_to_sync
    async def getGroupDefaultAccess(self, objectGuid:str) -> dict:
        return await self.network.request(
            method=f"getGroupDefaultAccess",
            input={"group_guid": objectGuid}
        )
    
    @async_to_sync
    async def setChatDefaultAccess(self, objectGuid:str, accessList:list) -> dict:
        chatType:str = Utils.getChatTypeByGuid(objectGuid=objectGuid)

        return await self.network.request(
            method=f"setGroupDefaultAccess",
            input={
                f"group_guid": objectGuid,
                "access_list": accessList
            }
        )
    
    @async_to_sync
    async def getGroupMentionList(self, objectGuid:str, searchMention:str) -> dict:
        return await self.network.request(
            method="getGroupMentionList",
            input={
                "group_guid": objectGuid,
                "search_mention": searchMention
            }
        )
    
    @async_to_sync
    async def editGroupInfo(
            self,
            objectGuid:str,
            title:Optional[str],
            description:Optional[str],
            slowMode:Optional[int],
            eventMessages:Optional[bool],
            chatHistoryForNewMembers:Optional[bool],
            reactionType:Optional[str], #Selected Disabled All
            selectedReactions:Optional[list[str]]
        ) -> dict:

        input:dict = {
            "group_guid": objectGuid,
            "updated_parameters": []
        }

        if title:
            input["title"] = title
            input["updated_parameters"].append("title")

        if description:
            input["description"] = description
            input["updated_parameters"].append("description")

        if slowMode:
            input["slow_mode"] = slowMode
            input["updated_parameters"].append("slow_mode")

        if not eventMessages is None:
            input["event_messages"] = eventMessages
            input["updated_parameters"].append("event_messages")

        if not chatHistoryForNewMembers is None:
            input["chat_history_for_new_members"] = "Visible" if chatHistoryForNewMembers else "Hidden"
            input["updated_parameters"].append("chat_history_for_new_members")

        if reactionType:
            if selectedReactions:
                input["chat_reaction_setting"] = {"reaction_type": reactionType,"selected_reactions":selectedReactions}
            else:
                input["chat_reaction_setting"] = {"reaction_type": reactionType}
            input["updated_parameters"].append("chat_reaction_setting")


        return await self.network.request(
            method="editGroupInfo",
            input=input
        )
    
    # Channel methods

    @async_to_sync
    async def addChannel(self, title:str, description:Optional[str], memberGuids:Optional[list], private:bool) -> dict:
        input:dict = {
            "title": title,
            "description": description,
            "member_guids": memberGuids or [],
            "channel_type": "Private" if private else "Public"
        }
        
        return await self.network.request(
            method="addChannel",
            input=input
        )
    
    @async_to_sync
    async def editChannelInfo(
            self,
            objectGuid:str,
            title:Optional[str],
            description:Optional[str],
            username:Optional[str],
            private:Optional[bool],
            signMessages:Optional[bool],
            reactionType:Optional[str], #Selected Disabled All
            selectedReactions:Optional[list]
        ) -> dict:

        input:dict = {
            "channel_guid": objectGuid,
            "updated_parameters": []
        }

        if selectedReactions: input["chat_reaction_setting"]["selected_reactions"] = selectedReactions

        if title:
            input["title"] = title
            input["updated_parameters"].append("title")

        if description:
            input["description"] = description
            input["updated_parameters"].append("description")

        if not private is None:
            input["channel_type"] = "Private" if private else "Public"
            input["updated_parameters"].append("channel_type")

        if not signMessages is None: 
            input["sign_messages"] = signMessages
            input["updated_parameters"].append("sign_messages")
        
        if reactionType:
            if selectedReactions:
                input["chat_reaction_setting"] = {"reaction_type": reactionType,"selected_reactions":selectedReactions}
            else:
                input["chat_reaction_setting"] = {"reaction_type": reactionType}
            input["updated_parameters"].append("chat_reaction_setting")

        if username:
            await self.updateChannelUsername(
                objectGuid=objectGuid,
                username=username
            )

        return await self.network.request(
            method="editChannelInfo",
            input=input
        )
    
    @async_to_sync
    async def checkChannelUsername(self, username:str) -> dict:
        return await self.network.request(
            method="checkChannelUsername",
            input={
                "username": username,
            }
        )
    
    @async_to_sync
    async def updateChannelUsername(self, objectGuid:str, username:str) -> dict:
        return await self.network.request(
            method="updateChannelUsername",
            input={
                "channel_guid": objectGuid,
                "username": username
            }
        )
    
    @async_to_sync
    async def getChannelSeenCount(self, objectGuid:str, minId:str, maxId:str) -> dict:
        return await self.network.request(
            method="getChannelSeenCount",
            input={
                "channel_guid": objectGuid,
                "min_id": minId,
                "max_id": maxId
            }
        )
    
    # Message methods
    @async_to_sync
    async def sendText(self, objectGuid:str, text:str, messageId:Optional[str]) -> dict:
        metadata = Utils.checkMetadata(text)

        input = {
            "object_guid": objectGuid,
            "rnd": str(randint(10000000, 999999999)),
            "text": metadata[1],
            "reply_to_message_id": messageId,
        }

        if metadata[0] != []: input["metadata"] = {"meta_data_parts": metadata[0]}

        return await self.network.request(method="sendMessage", input=input)
    
    @async_to_sync
    async def baseSendFileInline(
        self,
        objectGuid:str,
        file:str,
        text:Optional[str],
        messageId:Optional[str],
        fileName:Optional[str],
        type:Literal["Image", "Video", "Gif", "VideoMessage","Music", "Voice","File"],
        isSpoil:bool=False,
        customThumbInline:Optional[str] = None,
        time:Optional[int]=None,
        performer:Optional[str] = None
    ) -> Optional[dict]:
        upload_data = self.network.upload(file=file, fileName=fileName)
        if isinstance(upload_data,dict):
            uploadData:dict = dict(upload_data)
            if not uploadData:
                return
            
            input:dict = {
                "file_inline": {
                    "dc_id": uploadData["dc_id"],
                    "file_id": uploadData["id"],
                    "file_name": uploadData["file_name"],
                    "size": uploadData["size"],
                    "mime": uploadData["mime"],
                    "access_hash_rec": uploadData["access_hash_rec"],
                    "type": type,
                    "is_spoil": isSpoil
                },
                "object_guid": objectGuid,
                "rnd": Utils.randomRnd(),
                "reply_to_message_id": messageId
            }

            if type in ["Image", "Video", "Gif", "VideoMessage"]:
                customThumbInline = Utils.getImageThumbnail(
                    customThumbInline
                    if isinstance(customThumbInline, bytes)
                    else self.network.http.request("GET", customThumbInline).data
                    if Utils.checkLink(customThumbInline)
                    else open(customThumbInline, "rb").read()
                ) if customThumbInline else None

                videoData=[]
                if not type == "Image":
                    videoData:list = list(Utils.getVideoData(uploadData["file"]))
                    input["file_inline"]["time"] = videoData[2] * 1000

                fileSize:list = list(Utils.getImageSize(uploadData["file"])) if type == "Image" else videoData[1]
                input["file_inline"]["width"] = fileSize[0]
                input["file_inline"]["height"] = fileSize[1]

                if type == "VideoMessage":
                    input["file_inline"]["type"] = "Video"
                    input["file_inline"]["is_round"] = True

                input["file_inline"]["thumb_inline"] = customThumbInline or (Utils.getImageThumbnail(uploadData["file"]) if type == "Image" else videoData[0])

            if type in ["Music", "Voice"]:
                input["file_inline"]["time"] = (time or Utils.getVoiceDuration(uploadData["file"])) * (1000 if type == "Voice" else 1)

                if type == "Music":
                    input["file_inline"]["music_performer"] = performer or Utils.getMusicArtist(uploadData["file"])

            metadata:list = list(Utils.checkMetadata(text))
            if metadata[1]: input["text"] = metadata[1]
            if metadata[0]: input["metadata"] = {"meta_data_parts": metadata[0]}

            return await self.network.request(
                method="sendMessage",
                input=input
            )

    
    @async_to_sync
    async def sendFile(self, objectGuid:str, file:str, messageId:Optional[str], text:Optional[str], fileName:Optional[str] = None) -> Optional[dict]:
        if fileName is None:
            fileName = Utils.format_file("File")
        return await self.baseSendFileInline(
            objectGuid=objectGuid,
            file=file,
            text=text,
            messageId=messageId,
            fileName=fileName,
            type="File"
        )
    
    @async_to_sync
    async def sendImage(self, objectGuid:str, file:str, messageId:Optional[str], text:Optional[str], isSpoil:bool, thumbInline:Optional[str], fileName:Optional[str] = None) -> Optional[dict]:
        if fileName is None:
            fileName = Utils.format_file("Image")
        return await self.baseSendFileInline(
            objectGuid=objectGuid,
            file=file,
            text=text,
            messageId=messageId,
            fileName=fileName,
            type="Image",
            isSpoil=isSpoil,
            customThumbInline=thumbInline
        )
    
    @async_to_sync
    async def sendVideo(self, objectGuid:str, file:str, messageId:Optional[str], text:Optional[str], isSpoil:bool, thumbInline:Optional[str], fileName:Optional[str] = None) -> Optional[dict]:
        if fileName is None:
            fileName = Utils.format_file("Video")
        return await self.baseSendFileInline(
            objectGuid=objectGuid,
            file=file,
            text=text,
            messageId=messageId,
            fileName=fileName,
            type="Video",
            isSpoil=isSpoil,
            customThumbInline=thumbInline
        )
    
    @async_to_sync
    async def sendVideoMessage(self, objectGuid:str, file:str, messageId:Optional[str], text:Optional[str], thumbInline:Optional[str], fileName:Optional[str] = None) -> Optional[dict]:
        if fileName is None:
            fileName = Utils.format_file("Video")
        return await self.baseSendFileInline(
            objectGuid=objectGuid,
            file=file,
            text=text,
            messageId=messageId,
            fileName=fileName,
            type="VideoMessage",
            customThumbInline=thumbInline
        )
    
    @async_to_sync
    async def sendGif(self, objectGuid:str, file:str, messageId:Optional[str], text:Optional[str], thumbInline:Optional[str], fileName:Optional[str] = None) -> Optional[dict]:
        if fileName is None:
            fileName = Utils.format_file("Gif")
        return await self.baseSendFileInline(
            objectGuid=objectGuid,
            file=file,
            text=text,
            messageId=messageId,
            fileName=fileName,
            type="Gif",
            customThumbInline=thumbInline
        )
    
    @async_to_sync
    async def sendMusic(self, objectGuid:str, file:str, messageId:Optional[str], text:Optional[str], performer:Optional[str], fileName:Optional[str] = None) -> Optional[dict]:
        if fileName is None:
            fileName = Utils.format_file("Music")
        return await self.baseSendFileInline(
            objectGuid=objectGuid,
            file=file,
            text=text,
            messageId=messageId,
            fileName=fileName,
            type="Music",
            performer=performer
        )
    
    @async_to_sync
    async def sendVoice(self, objectGuid:str, file:str, time:int, messageId:Optional[str] = None, text:Optional[str] = None, fileName:Optional[str] = None) -> Optional[dict]:
        if fileName is None:
            fileName = Utils.format_file("Voice")
        return await self.baseSendFileInline(
            objectGuid=objectGuid,
            file=file,
            text=text,
            messageId=messageId,
            fileName=fileName,
            type="Voice",
            time=time
        )
    
    @async_to_sync
    async def sendLocation(self, objectGuid:str, latitude:int, longitude:int, messageId:Optional[str]) -> dict:
        return await self.network.request(
            method="sendMessage",
            input={
                "location": {
                    "latitude": latitude,
                    "longitude": longitude
                },
                "object_guid":objectGuid,
                "rnd": Utils.randomRnd(),
                "reply_to_message_id": messageId
            }
        )
    
    @async_to_sync
    async def sendMessageAPICall(self, objectGuid:str, text:str, messageId:str, buttonId:str) -> dict:
        return await self.network.request(
            method="sendMessageAPICall",
            input={
                "text": text,
                "object_guid": objectGuid,
                "message_id": messageId,
                "aux_data": {"button_id": buttonId}
            }
        )
    
    @async_to_sync
    async def editMessage(self, objectGuid, text, messageId) -> dict:
        metadata = Utils.checkMetadata(text)
        data = {
            "object_guid": objectGuid,
            "text": metadata[1],
            "message_id": messageId,
        }
        if metadata[0] != []:
            data["metadata"] = {"meta_data_parts": metadata[0]}
        return await self.network.request("editMessage", data)
    
    @async_to_sync
    async def actionOnMessageReaction(self, objectGuid:str, messageId:str, reactionId:Union[int,str], action:str) -> dict:
        if type(reactionId) is str:
            reactionId = Utils.reaction_to_id(reactionId)
        return await self.network.request(
            method="actionOnMessageReaction",
            input={
                "action": action, #Add OR Remove
                "object_guid": objectGuid,
                "message_id": messageId,
                "reaction_id": reactionId
            }
        )

    @async_to_sync
    async def setPinMessage(self, objectGuid:str, messageId:str, action:str) -> dict:
        return await self.network.request(
            method="setPinMessage",
            input={
                "object_guid": objectGuid,
                "message_id": messageId,
                "action": action
            }
        )

    @async_to_sync
    async def resendMessage(self, objectGuid:Optional[str], messageId:Optional[str], toObjectGuid:Optional[str], replyToMessageId:Optional[str], text:Optional[str], fileInline) -> dict:
        if not fileInline:
            messageData:dict = await self.getMessagesById(objectGuid=objectGuid, messageIds=[messageId])

        text_:str = text or messageData["messages"][0]["text"]
        metadata = Utils.checkMetadata(text_)
        input = {
            "is_mute": False,
            "object_guid": toObjectGuid,
            "rnd": Utils.randomRnd(),
            "reply_to_message_id": replyToMessageId,
            "text": metadata[1]
        }

        fileInline_:dict = fileInline or messageData["messages"][0]["file_inline"]
        if fileInline_:
            input["file_inline"] = fileInline_

        if not fileInline_:
            location:dict = messageData["messages"][0].get("location")
            if location:
                input["location"] = location
                del input["location"]["map_view"]
                del input["text"]

            contact:dict = messageData["messages"][0].get("message_contact")
            if contact:
                input["message_contact"] = contact
                del input["text"]

            sticker:dict = messageData["messages"][0].get("sticker")
            if sticker:
                input["sticker"] = sticker
                del input["text"]

            if messageData["messages"][0].get("metadata"):
                input["metadata"] = {"meta_data_parts": messageData["messages"][0]["metadata"]}
                
        elif metadata[0] != []:
            input["metadata"] = {"meta_data_parts": metadata[0]}

        return await self.network.request(method="sendMessage", input=input)

    @async_to_sync
    async def forwardMessages(self, objectGuid:str, messageIds:list, toObjectGuid:str) -> dict:
        return await self.network.request(
            method="forwardMessages",
            input={
                "from_object_guid": objectGuid,
                "message_ids": messageIds,
                "to_object_guid": toObjectGuid,
                "rnd": Utils.randomRnd(),
            }
        )
    
    @async_to_sync
    async def deleteMessages(self, objectGuid:str, messageIds:list, deleteForAll: bool = True) -> dict:
        return await self.network.request(
            method="deleteMessages",
            input={
                "object_guid": objectGuid,
                "message_ids": messageIds,
                "type": "Global" if deleteForAll else "Local"
            }
        )

    @async_to_sync
    async def autoDelete(self, objectGuid: str, messageId: str, time: int, deleteForAll: bool = True):
        await asyncio.sleep(time)
        return await self.deleteMessages(
            objectGuid=objectGuid,
            messageIds=[messageId],
            deleteForAll=deleteForAll
        )
    
    @async_to_sync
    async def getMessagesInterval(self, objectGuid:str, middleMessageId:str) -> dict:
        return await self.network.request(
            method="getMessagesInterval",
            input={
                "object_guid": objectGuid,
                "middle_message_id": middleMessageId
            }
        )
    
    @async_to_sync
    async def getMessages(self, objectGuid:str, maxId:Optional[str], filterType:Optional[str], limit:int) -> dict:
        input:dict = {
            "object_guid": objectGuid,
            "sort": "FromMax",
            "max_id": maxId,
            "limit": limit
        }
        
        if filterType: input["filter_type"] = filterType

        return await self.network.request(
            method="getMessages",
            input=input
        )
    
    @async_to_sync
    async def getMessagesUpdates(self, objectGuid:str) -> dict:
        return await self.network.request(
            method="getMessagesUpdates",
            input={
                "object_guid": objectGuid,
                "state": Utils.getState(),
            }
        )
    
    @async_to_sync
    async def getMessagesById(self, objectGuid:Optional[str], messageIds:list) -> dict:
        return await self.network.request(
            method="getMessagesByID",
            input={
                "object_guid": objectGuid,
                "message_ids": messageIds,
            }
        )
    
    @async_to_sync
    async def getMessageShareUrl(self, objectGuid:str, messageId:str) -> dict:
        return await self.network.request(
            method="getMessageShareUrl",
            input={
                "object_guid": objectGuid,
                "message_id": messageId,
            }
        )
    
    @async_to_sync
    async def clickMessageUrl(self, objectGuid:str, messageId:str, linkUrl:str) -> dict:
        return await self.network.request(
            method="clickMessageUrl",
            input={
                "object_guid": objectGuid,
                "message_id": messageId,
                "link_url": linkUrl
            }
        )
    
    @async_to_sync
    async def searchGlobalMessages(self, searchText:str) -> dict:
        return await self.network.request(
            method="search_text",
            input={
                "search_text": searchText,
                "type": "Text",
            }
        )
    
    @async_to_sync
    async def requestSendFile(self, fileName:str, mime:str, size:int) -> dict:
        return await self.network.request(
            method="requestSendFile",
            input={
                "file_name": fileName,
                "mime": mime,
                "size": str(size)
            }
        )
    
    # Contact methods
    
    @async_to_sync
    async def sendContact(self, objectGuid:str, firstName:str, lastName:str, phoneNumber:str, userGuid:str, messageId:Optional[str]) -> dict:
        return await self.network.request(
            method="sendMessage",
            input={
                "message_contact":{
                    "first_name": firstName,
                    "last_name": lastName or "",
                    "phone_number": phoneNumber,
                    "user_guid": userGuid
                },
                "object_guid":objectGuid,
                "rnd": Utils.randomRnd(),
                "reply_to_message_id": messageId
            }
        )

    @async_to_sync
    async def getContacts(self, startId:Optional[str]) -> dict:
        return await self.network.request(method="getContacts", input={"start_id": startId})
    
    @async_to_sync
    async def getContactsLastOnline(self, userGuids:list) -> dict:
        return await self.network.request(method="getContactsLastOnline", input={"user_guids": userGuids})

    @async_to_sync
    async def addAddressBook(self, phone:str, firstName:str, lastName:str) -> dict:
        return await self.network.request(
            method="addAddressBook",
            input={
                "phone": f"98{Utils.phoneNumberParse(phone)}",
                "first_name": firstName,
                "last_name": lastName
            }
        )
    
    @async_to_sync
    async def deleteContact(self, objectGuid:str) -> dict:
        return await self.network.request(method="deleteContact", input={"user_guid": objectGuid})
    
    @async_to_sync
    async def getContactsUpdates(self) -> dict:
        return await self.network.request(method="getContactsUpdates", input={"state": Utils.getState()})
    
    # Sticker methods

    @async_to_sync
    async def sendSticker(self, objectGuid:str, emoji:Optional[str], messageId:Optional[str], stickerData:Optional[str]) -> dict:
        data = {
            "sticker": (stickerData or choice((await self.getStickersByEmoji(emoji))["stickers"])) if emoji else None,
            "object_guid": objectGuid,
            "rnd": Utils.randomRnd(),
            "reply_to_message_id": messageId,
        }
        
        return await self.network.request("sendMessage", data)

    @async_to_sync
    async def getMyStickerSets(self) -> dict:
        return await self.network.request(method="getMyStickerSets")
    
    @async_to_sync
    async def getTrendStickerSets(self, startId:Optional[str]) -> dict:
        return await self.network.request(method="getTrendStickerSets", input={"start_id": startId})
    
    @async_to_sync
    async def searchStickers(self, searchText:str, startId:Optional[str]) -> dict:
        return await self.network.request(
            method="searchStickers",
            input={
                "search_text": searchText,
                "start_id": startId
            }
        )
    
    @async_to_sync
    async def actionOnStickerSet(self, stickerSetId:str, action:str) -> dict:
        return await self.network.request(
            method="actionOnStickerSet",
            input={
                "sticker_set_id": stickerSetId,
                "action": action
            }
        )
    
    @async_to_sync
    async def getStickersByEmoji(self, emoji:str) -> dict:
        return await self.network.request(
            method="getStickersByEmoji",
            input={
                "emoji_character": emoji,
                "suggest_by": "All"
            }
        )
    
    @async_to_sync
    async def getStickersBySetIDs(self, stickerSetIds:list) -> dict:
        return await self.network.request(method="getStickersBySetIDs", input={"sticker_set_ids": stickerSetIds})
    
    # Gif methods

    @async_to_sync
    async def getMyGifSet(self) -> dict:
        return await self.network.request(method="getMyGifSet")

    @async_to_sync
    async def addToMyGifSet(self, objectGuid:str, messageId:str) -> dict:
        return await self.network.request(
            method="addToMyGifSet",
            input={
                "message_id": messageId,
                "object_guid": objectGuid
            }
        )
    
    @async_to_sync
    async def removeFromMyGifSet(self, fileId:str) -> dict:
        return await self.network.request(method="removeFromMyGifSet", input={"file_id": fileId})
    
    # Poll methods

    @async_to_sync
    async def sendPoll(
        self,
        objectGuid: str,
        question: str,
        options: list,
        allowsMultipleResponses: bool = True,
        isAnonymous: bool = False,
        type: Literal["Quiz", "Regular"] = "Regular",
        messageId: Optional[str] = None,
        correctOptionIndex: Optional[int] = None,
        hint: Optional[str] = None
    ) -> dict:
        if not type in ['Quiz', 'Regular']:
            raise ValueError('type poll invalid ! type shoud is "Quiz" or "Regular".')
        if len(options) > 2:
            raise IndexError("Len for options is low ! Minimum for options is 2.")
        data = {
            'object_guid': objectGuid,
            'question': question,
            'options': options,
            'allows_multiple_answers': allowsMultipleResponses,
            'is_anonymous': isAnonymous,
            'reply_to_message_id': messageId,
            'type': type,
            'rnd': Utils.randomRnd(),
        }
        if type == 'Quiz':
            data['correct_option_index'] = correctOptionIndex
            data['explanation'] =  hint
        return await self.network.request(
            method="createPoll",
            input=data
        )
    
    @async_to_sync
    async def votePoll(self, pollId:str, selectionIndex:int) -> dict:
        return await self.network.request(
            method="votePoll",
            input={
                "poll_id": pollId,
                "selection_index": selectionIndex
            }
        )
    
    @async_to_sync
    async def getPollStatus(self, pollId:str) -> dict:
        return await self.network.request(method="getPollStatus", input={"poll_id": pollId})
    
    @async_to_sync
    async def getPollOptionVoters(self, pollId:str, selectionIndex:int, startId:Optional[str] = None) -> dict:
        return await self.network.request(
            method="getPollOptionVoters",
            input={
                "poll_id": pollId,
                "selection_index": selectionIndex,
                "start_id": startId
            }
        )
    
    # Live methods

    @async_to_sync
    async def sendLive(self, objectGuid:str, thumbInline:Union[bytes,str]) -> dict:
        if isinstance(thumbInline,bytes):
            by = thumbInline
        elif isinstance(thumbInline,str):
            if Utils.checkLink(thumbInline):
                by = self.network.http.request("GET", thumbInline)
                if not isinstance(by,bytes):
                    raise ValueError("Error !")
            else:
                with open(thumbInline,"rb") as file:
                    by = file.read()
        else:
            raise ValueError("thumbInline is byte or str !")
        return await self.network.request(
            method="sendLive",
            input={
                "thumb_inline": Utils.getImageThumbnail(by),
                "device_type": "Software",
                "object_guid": objectGuid,
                "rnd": Utils.randomRnd()
            }
        )
    
    @async_to_sync
    async def addLiveComment(self, accessToken:str, liveId:str, text:str) -> dict:
        return await self.network.request(
            method="addLiveComment",
            input={
                "access_token": accessToken,
                "live_id": liveId,
                "text": text
            }
        )
    
    @async_to_sync
    async def getLiveStatus(self, accessToken:str, liveId:str) -> dict:
        return await self.network.request(
            method="getLiveStatus",
            input={
                "access_token": accessToken,
                "live_id": liveId,
                "type": "LiveViewer"
            }
        )
    
    @async_to_sync
    async def getLiveComments(self, accessToken:str, liveId:str) -> dict:
        return await self.network.request(
            method="getLiveComments",
            input={
                "access_token": accessToken,
                "live_id": liveId,
            }
        )
    
    @async_to_sync
    async def getLivePlayUrl(self, accessToken:str, liveId:str) -> dict:
        return await self.network.request(
            method="getLivePlayUrl",
            input={
                "access_token": accessToken,
                "live_id": liveId
            }
        )
    
    # Call methods

    @async_to_sync
    async def requestCall(self, objectGuid:str, callType:str) -> dict:
        return await self.network.request(
            method="requestCall",
            input={
                "call_type": callType,
                "library_versions": ["2.7.7","2.4.4"],
                "max_layer": 92,
                "min_layer": 65,
                "sip_version": 1,
                "support_call_out": True,
                "user_guid": objectGuid
            }
        )
    
    @async_to_sync
    async def discardCall(self, callId:str, duration:int, reason:str) -> dict:
        return await self.network.request(
            method="discardCall",
            input={
                "call_id": callId,
                "duration": duration,
                "reason": reason #Missed OR Disconnect
            }
        )
    
    # Setting methods

    @async_to_sync
    async def setSetting(
            self,
            showMyLastOnline:Optional[bool],
            showMyPhoneNumber:Optional[bool],
            showMyProfilePhoto:Optional[bool],
            linkForwardMessage:Optional[bool],
            canJoinChatBy:Optional[bool]
        ) -> dict:

        input:dict = {
            "settings": {},
            "update_parameters": []
        }

        if not showMyLastOnline is None:
            input["settings"]["show_my_last_online"] = "Everybody" if showMyLastOnline else "Nobody"
            input["update_parameters"].append("show_my_last_online")

        if not showMyPhoneNumber is None:
            input["settings"]["show_my_phone_number"] = "Everybody" if showMyPhoneNumber else "Nobody"
            input["update_parameters"].append("show_my_phone_number")

        if not showMyProfilePhoto is None:
            input["settings"]["show_my_profile_photo"] = "Everybody" if showMyProfilePhoto else "MyContacts"
            input["update_parameters"].append("show_my_profile_photo")

        if not linkForwardMessage is None:
            input["settings"]["link_forward_message"] = "Everybody" if linkForwardMessage else "Nobody"
            input["update_parameters"].append("link_forward_message")

        if not canJoinChatBy is None:
            input["settings"]["can_join_chat_by"] = "Everybody" if canJoinChatBy else "MyContacts"
            input["update_parameters"].append("can_join_chat_by")

        return await self.network.request(
            method="setSetting",
            input=input
        )
    
    @async_to_sync
    async def addFolder(
            self,
            folderName:str,
            folderId:str,
            excludeChatIds:list,
            excludeChatTypes:list,
            includeChatIds:list,
            includeChatTypes:list
        ) -> dict:

        return await self.network.request(
            method="addFolder",
            input={
                "exclude_object_guids": excludeChatIds,
                "include_object_guids": excludeChatTypes,
                "exclude_chat_types": includeChatIds,
                "include_chat_types": includeChatTypes,
                "folder_id": folderId,
                "is_add_to_top": True,
                "name": folderName
            }
        )
    
    @async_to_sync
    async def getFolders(self, lastState:Optional[str]) -> dict:
        return await self.network.request(method="getFolders", input={"last_state": lastState})
    
    @async_to_sync
    async def getSuggestedFolders(self) -> dict:
        return await self.network.request(method="getSuggestedFolders")
    
    @async_to_sync
    async def deleteFolder(self, folderId:str) -> dict:
        return await self.network.request(method="deleteFolder", input={"folder_id": folderId})
    
    @async_to_sync
    async def updateProfile(self, firstName:Optional[str], lastname:Optional[str], bio:Optional[str], username:Optional[str]) -> dict:
        input:dict = {
            "first_name": firstName,
            "last_name": lastname,
            "bio": bio,
            "updated_parameters": []
        }

        if firstName:input["updated_parameters"].append("first_name")
        if lastname: input["updated_parameters"].append("last_name")
        if bio: input["updated_parameters"].append("bio")

        if username:
            response:dict = await self.network.request(method="updateUsername", input={"username": username})
            if not input["updated_parameters"]:
                return response

        return await self.network.request(
            method="updateProfile",
            input=input
        )
    
    @async_to_sync
    async def getMySessions(self) -> dict:
        return await self.network.request(method="getMySessions")
    
    @async_to_sync
    async def terminateSession(self, sessionKey:str) -> dict:
        return await self.network.request(method="terminateSession", input={"session_key": sessionKey})
    
    @async_to_sync
    async def terminateOtherSessions(self):
            return await self.network.request("terminateOtherSessions")
    
    @async_to_sync
    async def checkTwoStepPasscode(self, password:str) -> dict:
        return await self.network.request(method="checkTwoStepPasscode", input={"password": password})
    
    @async_to_sync
    async def setupTwoStepVerification(self, password:str, hint:str, recoveryEmail:str) -> dict:
        return await self.network.request(
            method="setupTwoStepVerification",
            input={
                "password": password,
                "hint": hint,
                "recovery_email": recoveryEmail
            }
        )
    
    @async_to_sync
    async def requestRecoveryEmail(self, password:str, recoveryEmail:str) -> dict:
        return await self.network.request(
            method="requestRecoveryEmail",
            input={
                "password": password,
                "recovery_email": recoveryEmail
            }
        )
    
    @async_to_sync
    async def verifyRecoveryEmail(self, password:str, code:str) -> dict:
        return await self.network.request(
            method="verifyRecoveryEmail",
            input={
                "password": password,
                "code": code
            }
        )
    
    @async_to_sync
    async def turnOffTwoStep(self, password:str) -> dict:
        return await self.network.request(method="turnOffTwoStep", input={"password": password})
    
    @async_to_sync
    async def changePassword(self, password:str, newPassword:str, newHint:str) -> dict:
        return await self.network.request(
            method="changePassword",
            input={
                "password": password,
                "new_password": newPassword,
                "new_hint": newHint
            }
        )
    
    @async_to_sync
    async def getTwoPasscodeStatus(self) -> dict:
        return await self.network.request(method="getTwoPasscodeStatus")
    
    @async_to_sync
    async def getPrivacySetting(self) -> dict:
        return await self.network.request(method="getPrivacySetting")
    
    @async_to_sync
    async def getBlockedUsers(self, startId:Optional[str]) -> dict:
        return await self.network.request(method="getBlockedUsers", input={"start_id": startId})
    
    # Other methods

    @async_to_sync
    async def getMe(self) -> dict:
        data:dict = await self.network.request(method="getUserInfo")
        data.update(self.sessionData)
        return data
    
    @async_to_sync
    async def transcribeVoice(self, objectGuid:str, messageId:str) -> dict:
        response = await self.network.request(
            method="transcribeVoice",
            input={
                "object_guid": objectGuid,
                "message_id": messageId
            }
        )
        
        if response["status"] != "OK":
            return response
        
        while True:
            sleep(0.5)
            result = await self.network.request(
                method="getTranscription",
                input={
                    "message_id": messageId,
                    "transcription_id": response["transcription_id"]
                }
            )
            
            if result["status"] != "OK":
                continue

            return result
    
    @async_to_sync
    async def resetContacts(self) -> dict:
        return await self.network.request("resetContacts")
    
    @async_to_sync
    async def getTime(self) -> dict:
        return await self.network.request("getTime")

    @async_to_sync
    async def getAbsObjects(self, objectGuids:list) -> dict:
        return await self.network.request(method="getAbsObjects", input={"object_guids": objectGuids})
    
    @async_to_sync
    async def getLinkFromAppUrl(self, url:str) -> dict:
        return await self.network.request(method="getLinkFromAppUrl", input={"app_url": url})
    
    @async_to_sync
    async def searchGlobalObjects(self, searchText:str, filters:Optional[list]) -> dict:
        input:dict = {"search_text": searchText}
        if filters: input["filter_types"] = filters

        return await self.network.request(method="searchGlobalObjects", input=input)
    
    @async_to_sync
    async def checkJoin(self, objectGuid:str, userGuid:str) -> Optional[bool]:
        userUsername:str = (await self.getChatInfo(userGuid))["user"].get("username")

        if userGuid in (await self.getChatAllMembers(objectGuid=objectGuid, searchText=userUsername, startId=None, justGetGuids=True)):
            return True
        
        if userUsername:
            return False
        
        return None
    
    @async_to_sync
    async def getProfileLinkItems(self, objectGuid:str) -> dict:
        return await self.network.request(method="getProfileLinkItems", input={"object_guid": objectGuid})
    
    @async_to_sync
    async def getDownloadLink(self, objectGuid:Optional[str], messageId:Optional[str], fileInline:Optional[dict]) -> Optional[str]:
        if not fileInline:
            fileInline = (await self.getMessagesById(objectGuid=objectGuid, messageIds=[messageId]))["messages"][0]["file_inline"]
        if fileInline:
            return f'https://messenger{fileInline["dc_id"]}.iranlms.ir/InternFile.ashx?id={fileInline["file_id"]}&ach={fileInline["access_hash_rec"]}'
    
    @async_to_sync
    async def download(self,save:bool, objectGuid:Optional[str] = None, messageId:Optional[str] = None, saveAs:Optional[str] = None, fileInline:Optional[dict] = None) -> Optional[dict]:
        if fileInline is None:
            fileInline = (await self.getMessagesById(objectGuid=objectGuid, messageIds=[messageId]))["messages"][0]["file_inline"]
        if not fileInline is None:
            downloading = self.network.download(
                accessHashRec=fileInline["access_hash_rec"],
                fileId=fileInline["file_id"],
                dcId=fileInline["dc_id"],
                size=fileInline["size"], 
                fileName=fileInline["file_name"]
            )

            if downloading:
                downloadedData:bytes = downloading

                if save or saveAs:
                    with open(saveAs or fileInline["file_name"], "wb") as file:
                        file.write(downloadedData)

                fileInline["file"] = downloadedData

                return fileInline

    @async_to_sync
    async def request(
        self,
        method: str,
        input: dict = {},
        tmpSession: bool = False,
        attempt: int = 0,
        maxAttempt: int = 2
    ) -> dict:
        return await self.network.request(method=method,input=input,tmpSession=tmpSession,attempt=attempt,maxAttempt=maxAttempt)

    @async_to_sync
    async def playVoice(self, objectGuid:str, file: str) -> None:
        try:
            from aiortc import RTCPeerConnection, RTCSessionDescription
            from aiortc.contrib.media import MediaPlayer
            voiceChatId: str = ((await self.getChatInfo(objectGuid))["chat"])["group_voice_chat_id"]

            if not voiceChatId:
                voiceChatId:str = (await self.createChatVoiceChat(objectGuid=objectGuid))["group_voice_chat_update"]["voice_chat_id"]

            rtcConnection = RTCPeerConnection()
            player: MediaPlayer = MediaPlayer(file)
            rtcConnection.addTrack(player.audio)
            spdOffer = await rtcConnection.createOffer()
            await rtcConnection.setLocalDescription(spdOffer)

            answerSdp = (await self.joinVoiceChat(
                objectGuid=objectGuid,
                myGuid=self.sessionData["user"]["user_guid"],
                voiceChatId=voiceChatId,
                sdp_offer_data=spdOffer.sdp
            ))["sdp_answer_data"]
            
            await self.setChatVoiceChatState(
                objectGuid=objectGuid,
                voideChatId=voiceChatId,
                activity="Unmute",
                participantObjectGuid=self.sessionData["user"]["user_guid"]
            )

            remoteDescription = RTCSessionDescription(answerSdp, "answer")
            await rtcConnection.setRemoteDescription(remoteDescription)

            def onEnded():
                asyncio.ensure_future(rtcConnection.close())

            player.audio.on("ended", onEnded)

            def keepAlive():
                while rtcConnection.connectionState != "closed":
                    try:
                        sending_code = asyncio.run(self.sendChatVoiceChatActivity(
                            objectGuid=objectGuid,
                            voideChatId=voiceChatId,
                            activity="Speaking",
                            participantObjectGuid=self.sessionData["user"]["user_guid"]
                        ))
                        if sending_code["status"] != "OK":
                            break
                        
                        if asyncio.run(self.getChatVoiceChatUpdates(
                            objectGuid=objectGuid,
                            voideChatId=voiceChatId
                        ))["status"] != "OK":
                            break

                        sleep(8)
                    except (InvalidInput, InvalidAuth):
                        break
                    except:
                        continue
                
            from threading import Thread
            Thread(target=keepAlive, daemon=False).start()

            while rtcConnection.connectionState != "closed":
                await asyncio.sleep(1)

            asyncio.ensure_future(rtcConnection.close())

        except ImportError:
            print("The aiortc library is not installed!")

    def add_handler(self, func, filters: Union[List[Filter], List[str], Filter, None]) -> None:
        self.socket.addHandler(
            func=func,
            filters=filters
        )
        return func
        
    def run(self) -> None:
        self.socket.connect()


from fast_rub import Client
from fast_rub.button import KeyPad
import asyncio

chat_id = "b..."
text = "test"

bot = Client("test")

async def test():
    sending = await bot.send_text(text, chat_id)
    buttons = KeyPad()
    buttons.add_1row().simple("100", "buttun 1")
    buttons.add_4row().simple("101", "buttun 1", "102", "buttun 2", "103", "buttun 3", "104", "buttun 4")
    sending2 = await bot.send_text(text, chat_id, inline_keypad=buttons.get())
    print(sending)
    print(sending2)

asyncio.run(test())


from fast_rub import Client
import asyncio

bot = Client("test")

chat_id = "b..."

async def test():
    list_foods = ["چلو قرمه سبزی", "زرشک پلو با مرغ"]
    sending = await bot.send_poll("chat id", "به چه غذایی علاقه دارید؟", list_foods)
    print(sending)

asyncio.run(test())


from fast_rub import Client
import asyncio

bot = Client("test")

async def test():
    await bot.add_commands("/help", "راهنمای ربات")
    sending = await bot.set_commands()
    print(sending)
    deleting = await bot.delete_commands()
    print(deleting)

asyncio.run(test())


from websocket import WebSocketApp
from .helper import Helper
from json import dumps, loads
from threading import Thread
from ..types import Message
from ..exceptions import NotRegistered, TooRequests
from ..utils import Utils
from re import match
from time import sleep
from typing import Optional , List, Union
import asyncio
from ..filters import Filter,legacy_filter

class Socket:
    def __init__(self, methods) -> None:
        self.methods = methods
        self.handlers = {}

    def connect(self) -> None:
        print("Connecting to the webSocket...")
        ws = WebSocketApp(
            Helper.getSocketServer(),
            on_open=self.onOpen,
            on_message=self.onMessage
        )
        ws.run_forever()

    def handShake(self, ws, data=None) -> None:
        ws.send(
            data or dumps(
                {
                    "auth": self.methods.sessionData["auth"],
                    "api_version": self.methods.apiVersion,
                    "method": "handShake"
                }
            )
        )

    def keepAlive(self, ws) -> None:
        while True:
            try:
                self.methods.getChatsUpdates()
                self.handShake(ws, "{}")
                sleep(30)
            except NotRegistered:
                raise
            except TooRequests:
                break
            except:
                continue

    def onOpen(self, ws) -> None:
        Thread(target=self.keepAlive, args=[ws]).start()
        self.handShake(ws)
        print("Connected.")

    def onMessage(self, _, message: str) -> None:
        if not message:
            return
        
        message: dict = loads(message)

        if not message.get("type") == "messenger":
            return
        
        message: dict = loads(self.methods.crypto.decrypt(message["data_enc"]))

        if not message.get("message_updates"):
            return

        for handler in self.handlers:
            filters: List[Filter] = self.handlers[handler]
            message_obj: Message = Message(message, self.methods)
            should_process = True
            for filter_obj in filters:
                if not filter_obj(message_obj):
                    should_process = False
                    break

            if should_process:
                Thread(target=lambda: asyncio.run(handler(message_obj))).start()


    def addHandler(self, func, filters: Union[List[Filter], List[str], Filter, None]) -> None:
        if filters and isinstance(filters, list) and all(isinstance(f, str) for f in filters):
            filters = [legacy_filter(filters)]
        elif isinstance(filters, Filter):
            filters = [filters]
        elif not filters:
            filters = []
        self.handlers[func] = filters
        return func


class Colors:
    BLACK = '\033[30m'
    RED = '\033[31m'
    GREEN = '\033[32m'
    YELLOW = '\033[33m'
    BLUE = '\033[34m'
    MAGENTA = '\033[35m'
    CYAN = '\033[36m'
    WHITE = '\033[37m'
    LIGHT_RED = '\033[91m'
    LIGHT_GREEN = '\033[92m'
    LIGHT_YELLOW = '\033[93m'
    LIGHT_BLUE = '\033[94m'
    LIGHT_MAGENTA = '\033[95m'
    LIGHT_CYAN = '\033[96m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    RESET = '\033[0m'
def cprint(text, color=Colors.WHITE, end='\n'):
    print(f"{color}{text}{Colors.RESET}", end=end)


from fast_rub import Client
from fast_rub.button import KeyPad
import asyncio

bot = Client("test")

chat_id = "b..."
message_id = "1234567890"

async def test():
    buttons = KeyPad()
    buttons.add_1row().simple("100", "this Key Pad inline is edited !")
    sending = await bot.edit_message_keypad_Inline(chat_id, message_id, buttons.get())
    print(sending)

asyncio.run(test())


from fast_rub import Client
from fast_rub.type import UpdateButton

bot = Client("test")

@bot.on_button()
async def test(msg:UpdateButton):
    print(msg)
    await msg.send_text("this is a text from fast rub")

bot.run()


from fast_rub import Client
import asyncio

bot = Client("test")

chat_id = "b..."
message_id = "1234567890"

async def test():
    sending = await bot.delete_message(chat_id, message_id)
    print(sending)

asyncio.run(test())


from fast_rub import Client
import asyncio

bot = Client("test")
url = "https://..."
type_url = "ReceiveUpdate"

async def test():
    update_end_point = await bot.set_endpoint(url, type_url)
    print(update_end_point)

asyncio.run(test())


import json
from typing import Optional

class metadata:
    def __init__(self,data:list) -> None:
        self.data = data
    def from_index(self,index:int) -> int:
        """from index / از اندیس"""
        return self.data[index]["from_index"]
    def length(self,index:int) -> int:
        """len / اندازه"""
        return self.data[index]["length"]
    def type(self,index:int) -> str:
        """type / نوع"""
        return self.data[index]["type"]
    def link_url(self,index:int) -> Optional[str]:
        """link / لینک"""
        return self.data[index]["link_url"] if "link_url" in self.data[index] else None
    def __str__(self) -> str:
        return json.dumps(self.data,indent=4,ensure_ascii=False)
    def __repr__(self) -> str:
        return self.__str__()


from fast_rub import Client
import asyncio

bot = Client("test")

chat_id = "b..."

async def test():
    sending = await bot.send_contact(chat_id, "first name", "last name", "+989017760881")
    print(sending)

asyncio.run(test())


from fast_rub import Client
from fast_rub.type import Update

bot = Client("test")

@bot.on_message()
async def test(msg:Update):
    print(msg)
    await msg.reply("this is a reply text from fast rub")

bot.run()


import json

class props:
    def __init__(self,data):
        self._data_ = data
    def __getattr__(self, name):
        return self.find_prop(keys=name)
    def __getitem__(self, key):
        return self._data_[key]
    def __lts__(self, update: list, *args, **kwargs):
        for index, element in enumerate(update):
            if isinstance(element, list):
                update[index] = self.__lts__(update=element)
            elif isinstance(element, dict):
                update[index] = props(data=element)
            else:
                update[index] = element
        return update
    def find_prop(self, keys, data_=None, *args, **kwargs):
        if data_ is None:
            data_ = self._data_
        if not isinstance(keys, list):
            keys = [keys]
        if isinstance(data_, dict):
            for key in keys:
                try:
                    update = data_[key]
                    if isinstance(update, dict):
                        update = props(data=update)
                    elif isinstance(update, list):
                        update = self.__lts__(update=update)
                    return update
                except KeyError:
                    pass
            data_ = data_.values()
        for value in data_:
            if isinstance(value, (dict, list)):
                try:
                    return self.find_prop(keys=keys, data_=value)
                except AttributeError:
                    return None
        return None
    def __str__(self) -> str:
        return json.dumps(self._data_,indent=4,ensure_ascii=False)
    @property
    def status(self):
        """status for requests / وضعیت درخواست"""
        return self._data_["status"]
    @property
    def is_ok_status(self):
        """is true status / درست بودن وضعیت"""
        return True if self.status == "OK" else False
    @property
    def data(self):
        """data (if has) / اطلاعات (اگر وجود داشته باشد)"""
        return self._data_["data"] if "data" in self._data_ else None


from typing import List
import re
import time as ti
from .types import Message
from .utils import Utils

class Filter:
    """Base filter class"""
    def __call__(self, message: Message) -> bool:
        raise NotImplementedError

class is_user(Filter):
    """filter by user / فیلتر با کاربر"""
    def __call__(self, message: Message) -> bool:
        return message.chat_type.lower() == "user"

class is_group(Filter):
    """filter by group / فیلتر با گروه"""
    def __call__(self, message: Message) -> bool:
        return message.chat_type.lower() == "group"

class is_channel(Filter):
    """filter by channel / فیلتر با کانال"""
    def __call__(self, message: Message) -> bool:
        return message.chat_type.lower() == "channel"

class is_bot(Filter):
    """filter by bot / فیلتر با بات"""
    def __call__(self, message: Message) -> bool:
        return message.chat_type.lower() == "bot"

class is_text(Filter):
    """filter by is text message / فیلتر با متن بودن پیام"""
    def __call__(self, message: Message) -> bool:
        return message.message_type.lower() == "text"

class is_image(Filter):
    """filter by is image message / فیلتر با تصویر بودن پیام"""
    def __call__(self, message: Message) -> bool:
        return message.message_type.lower() == "image"

class is_video(Filter):
    """filter by is video message / فیلتر با ویدیو بودن پیام"""
    def __call__(self, message: Message) -> bool:
        return message.message_type.lower() == "video"

class is_gif(Filter):
    """filter by is gif message / فیلتر با گیف بودن پیام"""
    def __call__(self, message: Message) -> bool:
        return message.message_type.lower() == "gif"

class is_voice(Filter):
    """filter by is voice message / فیلتر با ویس بودن پیام"""
    def __call__(self, message: Message) -> bool:
        return message.message_type.lower() == "voice"

class is_music(Filter):
    """filter by is music message / فیلتر با موزیک بودن پیام"""
    def __call__(self, message: Message) -> bool:
        return message.message_type.lower() == "music"

class is_file(Filter):
    """filter by is file message / فیلتر با فایل بودن پیام"""
    def __call__(self, message: Message) -> bool:
        return message.message_type.lower() == "file"

class is_sticker(Filter):
    """filter by is sticker message / فیلتر با استیکر بودن پیام"""
    def __call__(self, message: Message) -> bool:
        return message.message_type.lower() == "sticker"

class is_me(Filter):
    """filter by my messages / فیلتر با پیام های من"""
    def __call__(self, message: Message) -> bool:
        return message["chat_updates"]["chat"]["last_message"]["is_mine"]

class text(Filter):
    """filter text message by text /  فیلتر کردن متن پیام بر اساس متنی"""
    def __init__(self, pattern: str):
        self.pattern = pattern

    def __call__(self, message: Message) -> bool:
        return message.text == self.pattern

class regex(Filter):
    """filter text message by regex pattern / فیلتر متن پیام با regex"""
    def __init__(self, pattern: str, flags=0):
        self.pattern = re.compile(pattern, flags)

    def __call__(self, message: Message) -> bool:
        return bool(self.pattern.search(message.text or ""))

class commands(Filter):
    """filter text message by commands / فیلتر کردن متن پیام با دستورات"""
    def __init__(self, commands: List[str]):
        self.commands = [cmd.lower() for cmd in commands]

    def __call__(self, message: Message) -> bool:
        if not message.text:
            return False
        text = message.text.lower()
        return any(text == cmd or text.startswith(cmd + ' ') for cmd in self.commands)

class object_guid(Filter):
    """filter by object guid / فیلتر با آبجکت گوید"""
    def __init__(self, guid: str):
        self.guid = guid

    def __call__(self, message: Message) -> bool:
        return message.object_guid == self.guid

class object_guids(Filter):
    """filter by object guids / فیلتر با آبجکت گوید ها"""
    def __init__(self, guids: List[str]):
        self.guids = [guid.lower() for guid in guids]

    def __call__(self, message: Message) -> bool:
        return message.object_guid.lower() in self.guids

class author_guid(Filter):
    """filter by author guid / فیلتر با فرستنده گوید"""
    def __init__(self, guid: str):
        self.guid = guid

    def __call__(self, message: Message) -> bool:
        return message.author_guid == self.guid

class author_guids(Filter):
    """filter by author guids / فیلتر با فرستنده های گوید"""
    def __init__(self, guids: List[str]):
        self.guids = [guid.lower() for guid in guids]

    def __call__(self, message: Message) -> bool:
        return message.author_guid.lower() in self.guids

class time_range(Filter):
    """filter by time / فیلتر با زمان"""
    def __init__(self, from_time: float = 0, to_time: float = float("inf")):
        self.from_time = from_time
        self.to_time = to_time

    def __call__(self, message: Message) -> bool:
        message_time = message.time
        return self.from_time <= message_time <= self.to_time

class and_filter(Filter):
    """filters {and} for if all filters is True : run code ... / فیلتر های ورودی {and} که اگر تمامی فیلتر های ورودی برابر True بود اجرا شود"""
    def __init__(self, *filters):
        self.filters = filters

    def __call__(self, message: Message) -> bool:
        return all(f(message) for f in self.filters)

class or_filter(Filter):
    """filters {or} for if one filter is True : run code ... / فیلتر های ورودی {and} که اگر یک فیلتر ورودی برابر True بود اجرا شود"""
    def __init__(self, *filters):
        self.filters = filters

    def __call__(self, message: Message) -> bool:
        return any(f(message) for f in self.filters)

class not_filter(Filter):
    """filter by is False Filter / فیلتر با غلط بودن فیلتر"""
    def __init__(self, filter_obj):
        self.filter_obj = filter_obj

    def __call__(self, message: Message) -> bool:
        return not self.filter_obj(message)

class legacy_filter(Filter):
    """filter by chat type or message type (old filters PyRubi) / فیلتر با نوع چت یا نوع پیام (فیلتر های قدیمی پایروبی)"""
    def __init__(self, filters_list: List[str]):
        self.filters_list = [f.lower() for f in filters_list]
    def __call__(self, message: Message) -> bool:
        for filter_item in self.filters_list:
            if filter_item in ["user", "group", "channel", "bot"]:
                if message.chat_type.lower() == filter_item:
                    return True
            elif filter_item in ["text", "image", "video", "gif", "voice", "music", "file", "sticker"]:
                if message.message_type.lower() == filter_item:
                    return True
            elif Utils.getChatTypeByGuid(filter_item):
                if message.object_guid == filter_item:
                    return True
        return False


from fast_rub import Client
import asyncio

bot = Client("test")

chat_id = "b..."
text = "__Hello__ *from* **FastRub**"
text_HTML = "<b>test</b> <a href='https://rubika.ir'>rubika</a>"

async def test():
    sending = await bot.send_text(text,chat_id)
    print(sending)
    sending = await bot.send_text(text_HTML,chat_id,parse_mode="HTML")
    print(sending)

asyncio.run(test())


import asyncio
import inspect
from functools import wraps

def async_to_sync(func):
    """دکوراتور برای تبدیل تابع async به sync"""
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            loop = asyncio.get_running_loop()
            async def coro_wrapper():
                return await func(*args, **kwargs)
            return coro_wrapper()
        except RuntimeError:
            return asyncio.run(func(*args, **kwargs))
    return wrapper


def auto_async(func):
    """دکوراتور برای تبدیل تابع async به sync"""
    @wraps(func)
    def wrapper(*args, **kwargs):
        coro = func(*args, **kwargs)
        if not inspect.isawaitable(coro):
            return coro
        try:
            loop = asyncio.get_running_loop()
            in_co=inspect.currentframe()
            if in_co:
                f_co=in_co.f_back
                if f_co:
                    if f_co.f_back:
                        if f_co.f_code.co_flags & inspect.CO_COROUTINE:
                            return coro
            async def run_awaitable():
                return await coro
            asyncio.create_task(run_awaitable())
            return None
        except RuntimeError:
            async def run_wrapper():
                return await coro
            return asyncio.run(run_wrapper())
    return wrapper


from fast_rub import Client
import asyncio

bot = Client("test")

chat_id = "b..."
file_path = "test.json"
image_path = "test.png"

async def test():
    sending = await bot.send_file(chat_id, file_path, name_file="data.json", text="this is a caption")
    sending2 = await bot.send_image(chat_id, image_path, name_file="picture.png", text="this is a caption")
    print(sending)
    print(sending2)

asyncio.run(test())


import httpx
import os
import time
import json
import asyncio
import inspect
import aiofiles
from functools import wraps
from pathlib import Path
from typing import (
    Optional,
    Callable,
    Awaitable,
    Literal,
    Union,
    Dict,
    List,
    Any
)
from .utils.colors import *
from .filters import Filter
from .type import (
    Update,
    UpdateButton,
    msg_update
)
from .core.async_sync import *
from .utils.logger import *
from .type.props import *
from .utils.utils import (
    TextParser,
    utils
)
from .type.errors import (
    PollInvalid,
    CreateSessionError
)
from .utils.encryption import Encryption
from .network.network import Network


class Client:
    def __init__(
        self,
        name_session: str,
        token: Optional[str] = None,
        user_agent: Optional[str] = None,
        time_out: Optional[int] = 60,
        display_welcome = False,
        use_to_fastrub_webhook_on_message: Union[str,bool] = True,
        use_to_fastrub_webhook_on_button: Union[str,bool] = True,
        save_logs: Optional[bool] = None,
        view_logs: Optional[bool] = None,
        proxy: Optional[str] = None,
        main_parse_mode: Literal['Markdown', 'HTML', "Unknown", None] = "Unknown",
        max_retries: int = 3
    ):
        """Client for login and setting robot / کلاینت برای لوگین و تنظیمات ربات"""
        name = name_session + ".faru"
        self.name_session = name
        self.token = token
        self.time_out = time_out
        self.user_agent = user_agent
        self._running = False
        self.list_commands = []
        self.proxy = proxy
        self._fetch_messages = False
        self._fetch_messages_ = False
        self._fetch_buttons = False
        self._fetch_edit = False
        self._message_handlers = []
        self._button_handlers = []
        self._edit_handlers = []
        self._edit_handlers_ = []
        self.last = []
        self._message_handlers_ = []
        self.next_offset_id = ""
        self.next_offset_id_ = ""
        self.next_offset_id_get_message = ""
        self.geted_u = 0
        self.main_parse_mode: Literal['Markdown', 'HTML', 'Unknown', None] = main_parse_mode
        self.max_retries = max_retries
        self.list_befor_messages = []
        if os.path.isfile(name):
            with open(name, "r", encoding="utf-8") as file:
                encrypted_string = file.read().strip()
            try:
                decrypted = Encryption().de(encrypted_string)
                text_json_fast_rub_session = json.loads(decrypted)
                self.text_json_fast_rub_session = text_json_fast_rub_session
                self.token = text_json_fast_rub_session["token"]
                self.time_out = text_json_fast_rub_session["time_out"]
                self.user_agent = text_json_fast_rub_session["user_agent"]
                try:
                    self.log_to_file = text_json_fast_rub_session["setting_logs"]["save"]
                    self.log_to_console = text_json_fast_rub_session["setting_logs"]["view"]
                except:
                    pass
            except:
                cprint("Error for getting last data session !", Colors.RED)
                creating = utils.create_session(name_session,token,user_agent,time_out,display_welcome,view_logs,save_logs)
                if not creating:
                    raise CreateSessionError("Can Not Create The Session !")
        else:
            creating = utils.create_session(name_session,token,user_agent,time_out,display_welcome,view_logs,save_logs)
            if not creating:
                raise CreateSessionError("Can Not Create The Session !")
            self.token = token
            self.time_out = time_out
            self.user_agent = user_agent
        self.log_to_file = save_logs
        self.log_to_console = view_logs
        if self.log_to_file is None:
            self.log_to_file = False
        if self.log_to_console is None:
            self.log_to_console = False
        self.logger = logging.getLogger("fast_rub")
        self.network = Network(self.token,self.logger,self.max_retries,self.user_agent,proxy=proxy)
        self.use_to_fastrub_webhook_on_message = use_to_fastrub_webhook_on_message
        self.use_to_fastrub_webhook_on_button = use_to_fastrub_webhook_on_button
        if type(use_to_fastrub_webhook_on_message) is str:
            self._on_url = use_to_fastrub_webhook_on_message
        else:
            self._on_url = f"https://fast-rub.ParsSource.ir/geting_button_updates/get_on?token={self.token}"
        if type(use_to_fastrub_webhook_on_button) is str:
            self._button_url = use_to_fastrub_webhook_on_button
        else:
            self._button_url = f"https://fast-rub.ParsSource.ir/geting_button_updates/get?token={self.token}"
        self.urls = ["https://botapi.rubika.ir/v3/","https://messengerg2b1.iranlms.ir/v3/"]
        self.main_url = self.urls[0]
        try:
            self.version_botapi()
        except:
            self.main_url = self.urls[1]
        try:
            mes = self.get_updates(limit=100)
            self.next_offset_id_get_message = mes["data"]["next_offset_id"]
        except:
            pass
        setup_logging(log_to_console=self.log_to_console,log_to_file=self.log_to_file)
        if display_welcome:
            utils.print_time("Welcome to FastRub")
        self.logger.info("سشن اماده است")

    @property
    def TOKEN(self):
        self.logger.info("توکن دریافت شد")
        return self.token

    @async_to_sync  
    async def set_retry_settings(self, max_retries: int = 3):
        """تنظیمات تلاش مجدد"""
        self.max_retries = max_retries

    @async_to_sync
    async def version_botapi(self) -> str:
        """getting version botapi / گرفتن نسخه بات ای پی آی"""
        response = await self.network.request(self.main_url.replace("v3/",""),type_send="GET",timeout=self.time_out)
        version = response.text
        return version

    @async_to_sync
    async def set_logging(
        self,
        saving: Optional[bool] = None,
        viewing: Optional[bool] = None
    ):
        """on or off viewing and saveing logs / فعال یا غیرفعال کردن نمایش و ذخیره لاگ"""
        self.logger.info("استفاده از متود set_logging")
        if saving is None:
            saving = self.log_to_file
        if viewing is None:
            viewing = self.log_to_console
        try:
            self.text_json_fast_rub_session["setting_logs"]["save"] = saving
            self.text_json_fast_rub_session["setting_logs"]["view"] = viewing
        except:
            self.text_json_fast_rub_session["setting_logs"] = {
                "save":saving,
                "view":viewing
            }
        with open(self.name_session, "w") as fi:
            json.dump(self.text_json_fast_rub_session, fi, ensure_ascii=False, indent=4)
        if saving and viewing:
            self.logger = setup_logging(log_to_file=saving, log_to_console=viewing)
        self.logger.info(f"logging تنظیم شد | نمایش: {viewing} | ذخیره: {saving}")

    @async_to_sync
    async def set_timeout(self,time_out:int) -> None:
        """setting time out / تنظیم تایم اوت"""
        self.time_out = time_out

    @async_to_sync
    async def send_requests(
        self, method, data_: Optional[Union[Dict[str, Any], List[Any]]] = None
    ) -> dict:
        """send request to methods with retry mechanism"""
        response = await self.network.send_request(method=method,data_=data_)
        return response

    @async_to_sync
    async def auto_delete(self,chat_id:str,message_id:str,time_sleep:float) -> props:
        """auto delete message next {time_sleep} time s / حذف خودکار پیام بعد از فلان مقدار ثانیه"""
        await asyncio.sleep(time_sleep)
        result = await self.delete_message(chat_id,message_id)
        return props(result)

    @async_to_sync
    async def get_me(self) -> props:
        """geting info accont bot / گرفتن اطلاعات اکانت ربات
        Returns:
        dict: 
            - status (str): وضعیت درخواست (مثلا "OK")
            - data (dict): شامل اطلاعات ربات
                - bot (dict):
                    - bot_id (str): شناسه گوید ربات
                    - bot_title (str): نام نمایشی ربات
                    - description (str): توضیحات ربات
                    - username (str): نام کاربری ربات
                    - start_message (str): پیام شروع
                    - share_url (str): لینک اشتراک ربات"""
        self.logger.info("استفاده از متود get_me")
        result = await self.send_requests(method="getMe")
        return props(result)

    @async_to_sync
    async def set_main_parse_mode(self,parse_mode: Literal['Markdown', 'HTML', 'Unknown', None]) -> None:
        """setting parse mode main / تنظیم کردن مقدار اصلی پارس مود

توجه :
در صورت تغییر مارکدوان در کلاینت یا متود ست مین پارس مود , پارس مود همیشه روی آن حالت قرار میگیرد
در صورتی که میخواهید از این حالت خارج شود و از ورودی های متود ها پیروی کند مقدار آن را در متود ست مین پارس مود برابر 'Unknown' کنید"""
        self.main_parse_mode = parse_mode

    @async_to_sync
    async def parse_mode_text(self, text: str,parse_mode: Literal["Markdown","HTML","Unknown",None] = "Markdown") -> tuple:
        """setting parse mode text / تنظیم پارس مود متن"""
        if self.main_parse_mode != "Unknown":
            parse_mode = self.main_parse_mode
        if parse_mode == "Markdown":
            data = TextParser.checkMarkdown(text)
            return data
        elif parse_mode == "HTML":
            data = TextParser.checkHTML(text)
            return data
        return [], text

    async def _auto_delete(self, data: dict, auto_delete: Optional[int] = None):
        if auto_delete:
            message_id = data["data"]["message_id"]
            chat_id = data["chat_id"]
            await self.auto_delete(chat_id, message_id, auto_delete)

    @async_to_sync
    async def send_text(
        self,
        text: str,
        chat_id: str,
        inline_keypad: Optional[dict] = None,
        keypad: Optional[dict] = None,
        resize_keyboard: Optional[bool] = True,
        on_time_keyboard: Optional[bool] = False,
        disable_notification: Optional[bool] = False,
        reply_to_message_id: Optional[str] = None,
        auto_delete: Optional[int] = None,
        parse_mode: Literal["Markdown","HTML",None] = "Markdown"
    ) -> msg_update:
        """sending text to chat id / ارسال متنی به یک چت آیدی"""
        self.logger.info("استفاده از متود send_text")
        metadata, text  = await self.parse_mode_text(text, parse_mode)
        data = {
            "chat_id": chat_id,
            "text": text,
            "disable_notification": disable_notification,
            "reply_to_message_id": reply_to_message_id
        }
        if inline_keypad:
            data["inline_keypad"] = {"rows": inline_keypad}
        if metadata:
            data["metadata"] = {"meta_data_parts": metadata}
        if keypad:
            data["chat_keypad"] = {
                "rows": keypad,
                "resize_keyboard": resize_keyboard,
                "on_time_keyboard": on_time_keyboard,
            }
            data["chat_keypad_type"] = "New"
        result = await self.send_requests(
            "sendMessage",
            data,
        )
        result["data"]["chat_id"] = chat_id
        await asyncio.create_task(self._auto_delete(result, auto_delete))
        return msg_update(result, self)

    @async_to_sync
    async def send_message(
        self,
        chat_id: str,
        text: Optional[str] = None,
        inline_keypad: Optional[dict] = None,
        keypad: Optional[dict] = None,
        resize_keyboard: Optional[bool] = True,
        on_time_keyboard: Optional[bool] = False,
        disable_notification: bool = False,
        reply_to_message_id: Optional[str] = None,
        auto_delete: Optional[int] = None,
        parse_mode: Literal["Markdown","HTML",None] = "Markdown",
        # file
        file: Union[str , Path , bytes , None] = None,
        name_file: Optional[str] = None,
        type_file: Literal["File", "Image", "Voice", "Music", "Gif" , "Video"] = "File",
        file_id: Optional[str] = None,
        # poll
        question: Optional[str] = None,
        options: Optional[list] = None,
        type_poll: Literal["Regular", "Quiz"] = "Regular",
        is_anonymous: bool = True,
        correct_option_index: Optional[int] = None,
        allows_multiple_answers: bool = False,
        hint: Optional[str] = None,
        # location
        latitude: Optional[str] = None,
        longitude: Optional[str] = None,
        # contact
        first_name: Optional[str] = None,
        last_name: Optional[str] = None,
        phone_number: Optional[str] = None,
    ) -> Union[props, msg_update]:
        """send message / ارسال پیام"""
        if file_id:
            return await self.send_file_by_file_id(chat_id,file_id,text,reply_to_message_id,disable_notification,auto_delete,parse_mode)
        elif file:
            return await self.base_send_file(chat_id,file,name_file,text,reply_to_message_id,type_file,disable_notification,auto_delete,parse_mode)
        elif question != None and options != None:
            return await self.send_poll(chat_id,question,options,type_poll=type_poll,is_anonymous=is_anonymous,correct_option_index=correct_option_index,allows_multiple_answers=allows_multiple_answers,hint=hint,auto_delete=auto_delete,reply_to_message_id=reply_to_message_id,disable_notification=disable_notification)
        elif latitude != None and longitude != None:
            return await self.send_location(chat_id,latitude,longitude,disable_notification=disable_notification,reply_to_message_id=reply_to_message_id,auto_delete=auto_delete)
        elif first_name and last_name and phone_number:
            return await self.send_contact(chat_id,first_name,last_name,phone_number,reply_to_message_id=reply_to_message_id,disable_notificatio=disable_notification,auto_delete=auto_delete)
        elif text != None:
            return await self.send_text(text=text,chat_id=chat_id,inline_keypad=inline_keypad,disable_notification=disable_notification,reply_to_message_id=reply_to_message_id,auto_delete=auto_delete,parse_mode=parse_mode,keypad=keypad,on_time_keyboard=on_time_keyboard,resize_keyboard=resize_keyboard)
        raise ValueError("Please Write The Args !")

    @async_to_sync
    async def send_poll(
        self,
        chat_id: str,
        question: str,
        options: list,
        type_poll: Literal["Regular", "Quiz"] = "Regular",
        is_anonymous: bool = True,
        correct_option_index: Optional[int] = None,
        allows_multiple_answers: bool = False,
        hint: Optional[str] = None,
        disable_notification: bool = False,
        reply_to_message_id: Optional[str] = None,
        auto_delete: Optional[int] = None
    ) -> props:
        """sending poll to chat id / ارسال نظرسنجی به یک چت آیدی"""
        self.logger.info("استفاده از متود send_poll")
        if len(options) > 10:
            raise PollInvalid("len for options is logner from 10 option")
        data = {
            "chat_id": chat_id,
            "question": question,
            "options": options,
            "type": type_poll,
            "is_anonymous": is_anonymous,
            "correct_option_index": correct_option_index,
            "hint": hint,
            "allows_multiple_answers": allows_multiple_answers,
            "reply_to_message_id": reply_to_message_id,
            "disable_notification": disable_notification
        }
        result = await self.send_requests(
            "sendPoll",
            data,
        )
        result["data"]["chat_id"] = chat_id
        await asyncio.create_task(self._auto_delete(result, auto_delete))
        return props(result)

    @async_to_sync
    async def send_location(
        self,
        chat_id: str,
        latitude: str,
        longitude: str,
        chat_keypad : Optional[str] = None,
        disable_notification: Optional[bool] = False,
        reply_to_message_id: Optional[str] = None,
        chat_keypad_type: Optional[str] = None,
        auto_delete: Optional[int] = None
    ) -> props:
        """sending location to chat id / ارسال لوکیشن(موقعیت مکانی) به یک چت آیدی"""
        self.logger.info("استفاده از متود send_location")
        data = {
            "chat_id": chat_id,
            "latitude": latitude,
            "longitude": longitude,
            "chat_keypad": chat_keypad,
            "disable_notification": disable_notification,
            "reply_to_message_id": reply_to_message_id,
            "chat_keypad_type": chat_keypad_type,
        }
        result = await self.send_requests(
            "sendLocation",
            data,
        )
        result["data"]["chat_id"] = chat_id
        await asyncio.create_task(self._auto_delete(result, auto_delete))
        return props(result)

    @async_to_sync
    async def send_contact(
        self,
        chat_id: str,
        first_name: str,
        last_name: str,
        phone_number: str,
        chat_keypad : Optional[str] = None,
        chat_keypad_type: Optional[str] = None,
        inline_keypad: Optional[str] = None,
        reply_to_message_id: Optional[str] = None,
        disable_notificatio: Optional[bool] = False,
        auto_delete: Optional[int] = None
    ) -> props:
        """sending contact to chat id / ارسال مخاطب به یک چت آیدی"""
        self.logger.info("استفاده از متود send_contact")
        data = {
            "chat_id": chat_id,
            "first_name": first_name,
            "last_name": last_name,
            "phone_number": phone_number,
            "chat_keypad": chat_keypad,
            "disable_notificatio": disable_notificatio,
            "chat_keypad_type": chat_keypad_type,
            "inline_keypad": inline_keypad,
            "reply_to_message_id": reply_to_message_id,
        }
        result = await self.send_requests(
            "sendContact",
            data,
        )
        result["data"]["chat_id"] = chat_id
        await asyncio.create_task(self._auto_delete(result, auto_delete))
        return props(result)

    @async_to_sync
    async def get_chat(
        self,
        chat_id: str
    ) -> props:
        """geting info chat id info / گرفتن اطلاعات های یک چت"""
        self.logger.info("استفاده از متود get_chat")
        data = {"chat_id": chat_id}
        result = await self.send_requests(
            "getChat",
            data,
        )
        return props(result)

    @async_to_sync
    async def get_updates(self, limit : Optional[int] = None, offset_id : Optional[str] = None) -> props:
        """getting messages chats / گرفتن پیام های چت ها"""
        self.logger.info("استفاده از متود get_updates")
        data = {"offset_id": offset_id, "limit": limit}
        result = await self.send_requests(
            "getUpdates",
            data,
        )
        return props(result)

    @async_to_sync
    async def get_message(self,chat_id: str,message_id: str,limit_search: int = 100) -> Optional[Update]:
        updates = await self.get_updates(limit_search,self.next_offset_id_get_message)
        self.geted_u = len(updates["data"]["updates"])
        for message in updates["data"]["updates"]:
            if message["type"]=="NewMessage":
                if message["chat_id"] == chat_id and message['new_message']['message_id'] == message_id:
                    return Update(message,self)
        if self.geted_u >= 40:
            try:
                self.next_offset_id_get_message = updates["data"]["next_offset_id"]
                self.geted_u = 0
                return await self.get_message(chat_id,message_id,limit_search)
            except:
                pass
        return None

    @async_to_sync
    async def get_messages(self,chat_id: str,message_id: str,limit_search: int = 100,get_befor: int = 10) -> Optional[dict]:
        updates = await self.get_updates(limit_search,self.next_offset_id_get_message)
        result = {"messages":[]}
        self.geted_u = len(updates["data"]["updates"])
        for message in updates["data"]["updates"]:
            if message["type"]=="NewMessage" and message["chat_id"] == chat_id:
                self.list_befor_messages.append(message)
                if message['new_message']['message_id'] == message_id:
                    self.list_befor_messages.reverse()
                    for i in range(get_befor):
                        result['messages'].append(self.list_befor_messages[i])
                    return result
        if self.geted_u >= 40:
            try:
                self.next_offset_id_get_message = updates["data"]["next_offset_id"]
                self.geted_u = 0
                return await self.get_messages(chat_id,message_id,limit_search,get_befor)
            except:
                pass
        return result

    @async_to_sync
    async def forward_message(
        self,
        from_chat_id: str,
        message_id: str,
        to_chat_id: str,
        disable_notification : Optional[bool] = False,
        auto_delete: Optional[int] = None
    ) -> props:
        """forwarding message to chat id / فوروارد پیام به یک چت آیدی"""
        self.logger.info("استفاده از متود forward_message")
        data = {
            "from_chat_id": from_chat_id,
            "message_id": message_id,
            "to_chat_id": to_chat_id,
            "disable_notification": disable_notification,
        }
        result = await self.send_requests(
            "forwardMessage",
            data,
        )
        result["data"]["chat_id"] = to_chat_id
        await asyncio.create_task(self._auto_delete(result, auto_delete))
        return props(result)

    @async_to_sync
    async def forward_messages(
        self,
        from_chat_id: str,
        message_ids: list,
        to_chat_id: str,
        disable_notification : Optional[bool] = False,
        auto_delete: Optional[int] = None
    ) -> List[props]:
        """forwarding messages to chat id / فوروارد چند پیام به یک چت آیدی"""
        list_forwards = []
        for ms_id in message_ids:
            list_forwards.append(await self.forward_message(from_chat_id,ms_id,to_chat_id,disable_notification,auto_delete))
        return list_forwards

    @async_to_sync
    async def edit_message_text(
        self,
        chat_id: str,
        message_id: str,
        text: str,
        parse_mode: Literal["Markdown","HTML",None] = "Markdown"
    ) -> msg_update:
        """editing message text / ویرایش متن پیام"""
        self.logger.info("استفاده از متود edit_message_text")
        metadata, text  = await self.parse_mode_text(text, parse_mode)
        data = {"chat_id": chat_id, "message_id": message_id, "text": text}
        if metadata:
            data["metadata"] = {"meta_data_parts": metadata}
        result = await self.send_requests(
            "editMessageText",
            data,
        )
        result["data"]["chat_id"] = chat_id
        return msg_update(result, self)

    @async_to_sync
    async def delete_message(
        self,
        chat_id: str,
        message_id: str
    ) -> props:
        """delete message / پاکسازی(حذف) یک پیام"""
        self.logger.info("استفاده از متود delete_message")
        data = {"chat_id": chat_id, "message_id": message_id}
        result = await self.send_requests(
            "deleteMessage",
            data,
        )
        return props(result)

    @async_to_sync
    async def add_commands(self, command: str, description: str) -> None:
        """add command to commands list / افزودن دستور به لیست دستورات"""
        self.logger.info("استفاده از متود add_commands")
        self.list_commands.append(
            {"command": command.replace("/", ""), "description": description}
        )

    @async_to_sync
    async def set_commands(self) -> props:
        """set the commands for robot / تنظیم دستورات برای ربات"""
        self.logger.info("استفاده از متود set_commands")
        result = await self.send_requests(
            "setCommands",
            {"bot_commands": self.list_commands},
        )
        return props(result)

    @async_to_sync
    async def delete_commands(self) -> props:
        """clear the commands list / پاکسازی لیست دستورات"""
        self.logger.info("استفاده از متود delete_commands")
        self.list_commands = []
        result = await self.set_commands()
        return props(result)

    @async_to_sync
    async def edit_message_keypad_inline(
        self,
        chat_id: str,
        text: str,
        inline_keypad,
        disable_notification : Optional[bool] = False,
        reply_to_message_id: Optional[str] = None,
        parse_mode: Literal["Markdown","HTML",None] = "Markdown"
    ) -> props:
        """editing the text key pad inline / ویرایش پیام شیشه ای"""
        self.logger.info("استفاده از متود edit_message_keypad_Inline")
        metadata, text  = await self.parse_mode_text(text, parse_mode)
        data = {
            "disable_notification": disable_notification,
            "reply_to_message_id": reply_to_message_id,
            "chat_id": chat_id,
            "text": text,
            "inline_keypad": {"rows": inline_keypad},
        }
        if metadata:
            data["metadata"] = {"meta_data_parts": metadata}
        result = await self.send_requests(
            "editMessageText",
            data,
        )
        return props(result)
    
    @async_to_sync
    async def upload_file(self, url: str, file_name: str, file: Union[str , Path , bytes]) -> dict:
        """upload file to rubika server / آپلود فایل در سرور روبیکا"""
        self.logger.info("استفاده از متود upload_file")
        if isinstance(file, (bytes, bytearray)):
            d_file = {"file": (file_name, file, "application/octet-stream")}
        else:
            try:
                async with aiofiles.open(file, "rb") as fi:
                    fil = await fi.read()
                    d_file = {"file": (file_name, fil , "application/octet-stream")}
            except:
                file_ = (await self.network.request(str(file),type_send="GET")).content
                d_file = {"file":file_}
        async with httpx.AsyncClient(verify=False,proxy=self.proxy) as cl:
            response = await cl.post(url, files=d_file,timeout=9999)
            if response.status_code != 200:
                self.logger.error("خطا در آپلود فایل !")
                raise httpx.HTTPStatusError(
                    f"Request failed with status code {response.status_code}",
                    request=response.request,
                    response=response,
                )
            data = response.json()
            return data

    @async_to_sync
    async def send_file_by_file_id(
        self,
        chat_id: str,
        file_id: str,
        text: Optional[str] = None,
        reply_to_message_id: Optional[str] = None,
        disable_notification: Optional[bool] = None,
        auto_delete: Optional[int] = None,
        parse_mode: Literal["Markdown","HTML",None] = "Markdown"
    ) -> props:
        """sending file by file id / ارسال فایل با آیدی فایل"""
        self.logger.info("استفاده از متود send_file_by_file_id")
        metadata = []
        if text:
            metadata, text  = await self.parse_mode_text(text, parse_mode)
        data = {
            "chat_id": chat_id,
            "text": text,
            "file_id": file_id,
            "reply_to_message_id": reply_to_message_id,
            "disable_notification": disable_notification,
        }
        if metadata:
            data["metadata"] = {"meta_data_parts": metadata}
        sending = await self.send_requests("sendFile", data)
        sending["data"]["chat_id"] = chat_id
        await asyncio.create_task(self._auto_delete(sending, auto_delete))
        return props(sending)
    
    @async_to_sync
    async def request_send_file(
        self,
        type: Literal["File", "Image", "Voice", "Music", "Gif" , "Video"] = "File"
    ) -> dict:
        return await self.send_requests(
            "requestSendFile",
            {
                "type": type
            },
        )

    async def base_send_file(
        self,
        chat_id: str,
        file: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text : Optional[str] = None,
        reply_to_message_id : Optional[str] = None,
        type_file: Literal["File", "Image", "Voice", "Music", "Gif" , "Video"] = "File",
        disable_notification : Optional[bool] = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal["Markdown","HTML",None] = "Markdown"
    ) -> props:
        """sending file with types ['File', 'Image', 'Voice', 'Music', 'Gif' , 'Video'] / ارسال فایل با نوع های فایل و عکس و پیغام صوتی و موزیک و گیف و ویدیو"""
        self.logger.info("استفاده از متود base_send_file")
        request_file = await self.request_send_file(type_file)
        upload_url_file = request_file["data"]["upload_url"]
        if not name_file:
            name_file = utils.format_file(type_file)
        if not name_file:
            raise ValueError("type file is invalud !")
        upload_file = await self.upload_file(upload_url_file, name_file, file)
        file_id = upload_file["data"]["file_id"]
        send = await self.send_file_by_file_id(chat_id,file_id,text,reply_to_message_id,disable_notification,parse_mode=parse_mode,auto_delete=auto_delete)
        sended = send._data_
        sended["file_id"] = file_id
        sended["type_file"] = type_file
        if isinstance(file, (bytes, bytearray, memoryview)):
            sended["size_file"] = len(file)
        elif isinstance(file, (str, Path)):
            try:
                async with aiofiles.open(file, "rb") as fi:
                    fil = await fi.read()
                    size_file = len(fil)
            except:
                size_file = len((await self.network.request(str(file))).content)
                sended["size_file"] = size_file
        else:
            raise FileExistsError("file not found !")
        return props(sended)

    @async_to_sync
    async def send_file(
        self,
        chat_id: str,
        file: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text : Optional[str] = None,
        reply_to_message_id : Optional[str] = None,
        disable_notification: Optional[bool] = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal["Markdown","HTML",None] = "Markdown"
    ) -> props:
        "ارسال فایل / send file"
        self.logger.info("استفاده از متود send_file")

        result = await self.base_send_file(
            chat_id,
            file,
            name_file,
            text,
            reply_to_message_id,
            "File",
            disable_notification,
            auto_delete,
            parse_mode
        )
        return props(result)

    @async_to_sync
    async def send_document(
        self,
        chat_id: str,
        file: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text : Optional[str] = None,
        reply_to_message_id : Optional[str] = None,
        disable_notification: Optional[bool] = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal["Markdown","HTML",None] = "Markdown"
    ) -> props:
        "ارسال فایل / send file"
        self.logger.info("استفاده از متود send_document")

        result = await self.base_send_file(
            chat_id,
            file,
            name_file,
            text,
            reply_to_message_id,
            "File",
            disable_notification,
            auto_delete,
            parse_mode
        )
        return props(result)

    @async_to_sync
    async def send_image(
        self,
        chat_id: str,
        image: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text : Optional[str] = None,
        reply_to_message_id : Optional[str] = None,
        disable_notification: Optional[bool] = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal["Markdown","HTML",None] = "Markdown"
    ) -> props:
        """sending image / ارسال تصویر"""
        self.logger.info("استفاده از متود send_image")
        
        result = await self.base_send_file(
            chat_id,
            image,
            name_file,
            text,
            reply_to_message_id,
            "Image",
            disable_notification,
            auto_delete,
            parse_mode
        )
        return props(result)

    @async_to_sync
    async def send_video(
        self,
        chat_id: str,
        video: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text : Optional[str] = None,
        reply_to_message_id : Optional[str] = None,
        disable_notification : Optional[bool] = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal["Markdown","HTML",None] = "Markdown"
    ) -> props:
        """sending video / ارسال ویدیو"""
        self.logger.info("استفاده از متود send_video")
        result = await self.base_send_file(
            chat_id,
            video,
            name_file,
            text,
            reply_to_message_id,
            "Video",
            disable_notification,
            auto_delete,
            parse_mode
        )
        return props(result)

    @async_to_sync
    async def send_voice(
        self,
        chat_id: str,
        voice: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text : Optional[str] = None,
        reply_to_message_id: Optional[str] = None,
        disable_notification: Optional[bool] = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal["Markdown","HTML",None] = "Markdown"
    ) -> props:
        """sending voice / ارسال ویس"""
        self.logger.info("استفاده از متود send_voice")
        result = await self.base_send_file(
            chat_id,
            voice,
            name_file,
            text,
            reply_to_message_id,
            "Voice",
            disable_notification,
            auto_delete,
            parse_mode
        )
        return props(result)

    @async_to_sync
    async def send_music(
        self,
        chat_id: str,
        music: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text : Optional[str] = None,
        reply_to_message_id : Optional[str] = None,
        disable_notification : Optional[bool] = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal["Markdown","HTML",None] = "Markdown"
    ) -> props:
        """sending music / ارسال موزیک"""
        self.logger.info("استفاده از متود send_music")
        result = await self.base_send_file(
            chat_id,
            music,
            name_file,
            text,
            reply_to_message_id,
            "Music",
            disable_notification,
            auto_delete,
            parse_mode
        )
        return props(result)

    @async_to_sync
    async def send_gif(
        self,
        chat_id: str,
        gif: Union[str , Path , bytes],
        name_file: Optional[str] = None,
        text : Optional[str] = None,
        reply_to_message_id : Optional[str] = None,
        disable_notification : Optional[bool] = False,
        auto_delete: Optional[int] = None,
        parse_mode: Literal["Markdown","HTML",None] = "Markdown"
    ) -> props:
        """sending gif / ارسال گیف"""
        self.logger.info("استفاده از متود send_gif")
        result = await self.base_send_file(
            chat_id,
            gif,
            name_file,
            text,
            reply_to_message_id,
            "Gif",
            disable_notification,
            auto_delete,
            parse_mode
        )
        return props(result)

    @async_to_sync
    async def send_sticker(
        self,
        chat_id: str,
        id_sticker: str,
        reply_to_message_id : Optional[str] = None,
        disable_notification : Optional[bool] = False,
        auto_delete: Optional[int] = None
    ) -> props:
        """sending sticker by id / ارسال استیکر با آیدی"""
        self.logger.info("استفاده از متود send_sticker")
        data = {
            "chat_id": chat_id,
            "sticker_id": id_sticker,
            "reply_to_message_id": reply_to_message_id,
            "disable_notification": disable_notification
        }
        sender = await self.send_requests("sendSticker", data)
        await asyncio.create_task(self._auto_delete(sender, auto_delete))
        return props(sender)

    @async_to_sync
    async def get_download_file_url(self,id_file : str) -> str:
        """get download url file / گرفتن آدرس دانلود فایل"""
        self.logger.info("استفاده از متود get_download_file_url")
        data = {"file_id": id_file}
        url = (await self.send_requests("getFile",data))["download_url"]
        return url

    @async_to_sync
    async def download_by_url(self, url: str, path: str = "file") -> None:
        download = await self.network.download(url, path)
        if download:
            self.logger.info("فایل دانلود شد")
        else:
            self.logger.error("خطا در دانلود فایل !")

    @async_to_sync
    async def download_file(self,id_file : str , path : str = "file") -> None:
        """download file / دانلود فایل"""
        self.logger.info("استفاده از متود download_file")
        url = await self.get_download_file_url(id_file)
        await self.download_by_url(url, path)

    @async_to_sync
    async def set_endpoint(self, url: str, type: Literal["ReceiveUpdate", "GetSelectionItem", "ReceiveInlineMessage", "ReceiveQuery", "SearchSelectionItems"]) -> dict:
        """set endpoint url / تنظیم ادرس اند پوینت"""
        self.logger.info("استفاده از متود set_endpoint")
        result = await self.send_requests(
            "updateBotEndpoints", {"url": url, "type": type}
        )
        return result
    
    @async_to_sync
    async def update_end_point(self, url: str, type: Literal["ReceiveUpdate", "GetSelectionItem", "ReceiveInlineMessage", "ReceiveQuery", "SearchSelectionItems"]) -> dict:
        """set endpoint url / تنظیم ادرس اند پوینت"""
        return await self.set_endpoint(url, type)

    def _schedule_handler(self, handler, update):
        async def _wrapped():
            try:
                await handler(update)
            except Exception:
                self.logger.exception("Error in handler")
        asyncio.create_task(_wrapped())

    @async_to_sync
    async def set_token_fast_rub(self) -> bool:
        """seting token in fast_rub for getting click glass messages and updata messges / تنظیم توکن در فست روب برای گرفتن کلیک های روی پیام شیشه ای و آپدیت پیام ها"""
        self.logger.info("استفاده از متود set_token_fast_rub")
        try:
            check_setted = await self.network.request(f"https://fast-rub.ParsSource.ir/set_token?token={self.token}")
            check_setted = check_setted.json()
            list_getted: List[Literal["ReceiveUpdate", "ReceiveInlineMessage"]] = ["ReceiveUpdate", "ReceiveInlineMessage"]
            for get in list_getted:
                url = f"https://fast-rub.ParsSource.ir/geting_button_updates/{self.token}/{get}"
                await self.set_endpoint(url, get)
            return True
        except:
            return False

    def on_message(self, filters: Optional[Filter] = None):
        """برای دریافت پیام‌های معمولی"""
        self._fetch_messages_ = True
        def decorator(handler):
            @wraps(handler)
            async def wrapped(update):
                if filters is None or filters(update):
                    if inspect.iscoroutinefunction(handler):
                        return await handler(update)
                    else:
                        return handler(update)
            self._message_handlers_.append(wrapped)
            return handler
        return decorator

    async def _process_messages_(self, time_updata_sleep: Union[float, float] = 0.5):
        while self._running:
            try:
                mes = await self.get_updates(limit=100, offset_id=self.next_offset_id)
                
                if mes.status == "INVALID_ACCESS":
                    raise PermissionError("Due to Rubika's restrictions, access to retrieve messages has been blocked.")
                    
                try:
                    self.next_offset_id = mes["data"]["next_offset_id"]
                except:
                    pass
                    
                for message in mes["data"]["updates"]:
                    if message["type"] == "NewMessage":
                        time_sended_mes = int(message['new_message']['time'])
                        now = int(time.time())
                        time_ = time_updata_sleep + 4
                        
                        if (now - time_sended_mes < time_) and (not message['new_message']['message_id'] in self.last):
                            self.last.append(message['new_message']['message_id'])
                            if len(self.last) > 500:
                                self.last.pop(0)
                            update_obj = Update(message, self)
                            for handler in self._message_handlers_:
                                self._schedule_handler(handler, update_obj)
                                
                await asyncio.sleep(time_updata_sleep)
                
            except (httpx.ReadError, httpx.ConnectError) as e:
                self.logger.warning(f"خطای شبکه در _process_messages_: {e} - انتظار 5 ثانیه...")
                await asyncio.sleep(5)
            except Exception as e:
                self.logger.error(f"خطای ناشناخته در _process_messages_: {e}")
                await asyncio.sleep(10)

    def on_message_updates(self, filters: Optional[Filter] = None):
        """برای دریافت آپدیت‌های پیام"""
        self._fetch_messages = True
        def decorator(handler):
            @wraps(handler)
            async def wrapped(update):
                if filters is None or filters(update):
                    if inspect.iscoroutinefunction(handler):
                        return await handler(update)
                    else:
                        return handler(update)
            self._message_handlers.append(wrapped)
            return handler
        return decorator

    def on_button(self):
        self._fetch_buttons = True
        def decorator(handler):
            @wraps(handler)
            async def wrapped(update):
                try:
                    if inspect.iscoroutinefunction(handler):
                        await handler(update)
                    else:
                        handler(update)
                except Exception as e:
                    self.logger.exception(f"Error in button handler: {e}")
            self._button_handlers.append(wrapped)
            return handler
        return decorator

    def on_edit_updates(self):
        """برای دریافت ویرایش شدن پیام ها"""
        self._fetch_edit = True
        def decorator(handler: Callable[[Update], Awaitable[None]]):
            self._edit_handlers_.append(handler)
            return handler
        return decorator

    async def _process_messages(self):
        while self._running:
            response = (await self.network.request(self._on_url, timeout=self.time_out)).json()
            if response and response.get('status') is True:
                results = response.get('updates', [])
                if results:
                    for result in results:
                        if result["type"] != "NewMessage":
                            continue
                        update = Update(result,self)
                        for handler in self._message_handlers:
                            self._schedule_handler(handler,update)
            else:
                await self.set_token_fast_rub()
            await asyncio.sleep(0.1)

    async def _process_edit_updates(self):
        while self._running:
            response = (await self.network.request(self._button_url, timeout=self.time_out)).json()
            if response and response.get('status') is True:
                results = response.get('updates', [])
                if results:
                    for result in results:
                        if result["type"] != "UpdatedMessage":
                            continue
                        update = UpdateButton(result,self)
                        for handler in self._edit_handlers:
                            self._schedule_handler(handler,update)
            else:
                await self.set_token_fast_rub()
            await asyncio.sleep(0.1)

    async def _fetch_button_updates(self):
        while self._running:
            response = (await self.network.request(self._button_url, timeout=self.time_out)).json()
            if response and response.get('status') is True:
                results = response.get('updates', [])
                if results:
                    for result in results:
                        update = UpdateButton(result,self)
                        for handler in self._button_handlers:
                            self._schedule_handler(handler,update)
            else:
                await self.set_token_fast_rub()
            await asyncio.sleep(0.1)

    async def _run_all(self):
        tasks = []
        if self._fetch_buttons:
            tasks.append(self._fetch_button_updates())
        if self._fetch_messages:
            tasks.append(self._process_messages())
        if self._fetch_messages_ and self._message_handlers_:
            tasks.append(self._process_messages_())
        if self._fetch_edit and self._edit_handlers_:
            tasks.append(self._process_edit_updates())
        if not tasks:
            raise ValueError("No handlers registered. Use on_message() or on_message_updates() or on_button() or on_edit_updates() first.")
        await asyncio.gather(*tasks)

    def run(self):
        """اجرای اصلی بات - فقط اگر هندلرهای مربوطه ثبت شده باشند"""
        if not (self._fetch_messages or self._fetch_buttons or self._fetch_messages_ or self._fetch_edit):
            raise ValueError("No update types selected. Use on_message() or on_message_updates() or on_button() or on_edit_updates() first.")
        
        if (self._fetch_messages and not self._message_handlers) or (self._fetch_messages_ and not self._message_handlers_):
            raise ValueError("Message handlers registered but no message callbacks defined.")
        
        if self._fetch_buttons and not self._button_handlers:
            raise ValueError("Button handlers registered but no button callbacks defined.")

        if self._fetch_edit and not self._edit_handlers_:
            raise ValueError("Edit handlers registered but no message callbacks defined.")

        self._running = True
        self.logger.info("ربات در حال دریافت پیام ها")
        utils.print_time("Start")
        asyncio.run(self._run_all())

    def stop(self):
        """خاموش کردن گرفتن آپدیت ها / off the getting updates"""
        self._running = False
